package sgraphics;

// Copyright (c) 1987-91 Stanford University
// Copyright (c) 1991-95 Silicon Graphics, Inc.
// Copyright (c) 1993-95 2001 S.A.
// Copyright (c) 1993-96 Fujitsu, Ltd.
// Copyright (c) 1994-96 Firefly Design
// Copyright (c) 1995-96 Vitria, Inc.
// Copyright (c) 1996    Michael A. Jones

import java.util.Observable;
import sgraphics.util.*;

/**
 * A area is a two dimensional area.
 */
public class Area extends Observable implements Axis {

	private boolean xDefined_;
	private boolean yDefined_;
	private Point lower_ = new Point();
	private Point upper_= new Point();
	private float xAlign_;
	private float yAlign_;

	/**
	 * Initializes an empty bounding box with all coordinates zero.
	 */
	public Area () {
		xDefined_ = false;
		yDefined_ = false;
		lower_.x = lower_.y = 0F;
		upper_.x = upper_.y = 0F;
		xAlign_ = yAlign_ = 0F;
	}

	/**
	 * Initializes a bounding box.
	 */
	public Area (float x1, float y1, float xalign, float x2, float y2, float yalign) {
		xDefined_ = true;
		yDefined_ = true;
		lower_.x = x1;
		lower_.y = y1;
		upper_.x = x2;
		upper_.y = y2;
		xAlign_ = xalign;
		yAlign_ = yalign;
		}

	/**
	 * Initializes a bounding box.
	 */
	public Area (float x1, float y1, float xalign, boolean xDefined, float x2, float y2, float yalign, boolean yDefined) {
		xDefined_ = xDefined;
		yDefined_ = yDefined;
		lower_.x = x1;
		lower_.y = y1;
		upper_.x = x2;
		upper_.y = y2;
		xAlign_ = xalign;
		yAlign_ = yalign;
		}

	/**
	 * Construct a area from a given region.
	 */
	public Area (Area area) {
		xDefined_ = area.xDefined_;
		yDefined_ = area.yDefined_;
		lower_.x = area.lower_.x;
		lower_.y = area.lower_.y;
		upper_.x = area.upper_.x;
		upper_.y = area.upper_.y;
		xAlign_ = area.xAlign_;
		yAlign_ = area.yAlign_;
	}

	/**
	 * Determine whether the x dimension is defined..
	 */
	public boolean xDefined () {
		return xDefined_;
	}

	/**
	 * Determine whether y dimension is defined.
	 */
	public boolean yDefined () {
		return yDefined_;
	}

	/**
	 * Determine if a point intersects the area.
	 *
	 * @param p the point.
	 * @return true if there is an intersection.
	 */
	public boolean contains (Point p) {
		return xDefined_ && yDefined_ &&
			p.x >= lower_.x && p.x <= upper_.x &&
			p.y >= lower_.y && p.y <= upper_.y;
		}

	/**
	 * Determine if a area intersects an area.
	 *
	 * @param area the area.
	 * @return true if there is an intersection.
	 */
	public boolean intersects (Area area) {
		boolean b = false;
		if (xDefined_ && yDefined_) {
			Point [] bounds = area.bounds();
			Point lower = bounds[0];
			Point upper = bounds[1];
			b = upper.x >= lower_.x && lower.x <= upper_.x &&
				upper.y >= lower_.y && lower.y <= upper_.y;
		}
		return b;
	}

	/**
	 * Copy the given area into this area.  This operation
	 * is equivalent to setting this area to the empty area and
	 * then calling mergeUnion on the given area.
	 */
	public void copy (Area area) {
		BoundingSpan x = new BoundingSpan();
		BoundingSpan y = new BoundingSpan();
		x = area.span(xAxis);
		y = area.span(yAxis);
		xDefined_ = area.xDefined();
		yDefined_ = area.yDefined();
		lower_.x = x.begin;
		lower_.y = y.begin;
		upper_.x = x.end;
		upper_.y = y.end;
		xAlign_ = x.align;
		yAlign_ = y.align;
		setChanged();
		notifyObservers();
	}

	/**
	 * Sets the area to be the intersection of itself with the
	 * given area. The origin of the area is unchanged.
	 */
	public void mergeIntersect (Area area) {
			Point bounds[] = area.bounds();
			Point lower = bounds[0];
			Point upper = bounds[1];
			mergeMax(lower_, lower);
			mergeMin(upper_, upper);

			setChanged();
			notifyObservers();
	}

	/**
	 * Sets the area to union of itself with the
	 * given area. The origin of the area is unchanged.
	 */
	public void mergeUnion (Area area) {
			Point bounds[] = area.bounds();
			Point lower = bounds[0];
			Point upper = bounds[1];
			mergeMin(lower_, lower);
			mergeMax(upper_, upper);

			setChanged();
			notifyObservers();
	}

	/**
	 * Get upper and lower coordinates.
	 * @return two points in an array.
	 */
	public Point[] bounds () {
		Point b[] = new Point[2];
		b[0] = lower_;
		b[1] = upper_;
		return b;
		}

	/**
	 * Get the center of the area.
	 */
	public Point center () {
		Point c = new Point((lower_.x + upper_.x) * 0.5F, (lower_.y + upper_.y) * 0.5F);
		return c;
	}

	/**
	 * Get the origin of the area.
	 */
	public Point origin () {
		Point o = new Point(spanOrigin(lower_.x, upper_.x, xAlign_), spanOrigin(lower_.y, upper_.y, yAlign_));
		return o;
	}

	/**
	 * Get the span of particular axis.
	 * @param axis the axis. (xAxis or yAxis)
	 */
	public BoundingSpan span (int axis) {
		BoundingSpan s = new BoundingSpan();

		switch (axis) {
			case xAxis :
				s.begin = lower_.x;
				s.end = upper_.x;
				s.align = xAlign_;
				break;
			case yAxis :
				s.begin = lower_.y;
				s.end = upper_.y;
				s.align = yAlign_;
				break;
		}
		s.length = s.end - s.begin + 1;
		s.origin = s.begin + s.align * (s.length - 1);
		return s;
	}

	private void mergeMin (Point p1, Point p2) {
		p1.x = Math.min(p1.x, p2.x);
		p1.y = Math.min(p1.y, p2.y);
	}

	private void mergeMax (Point p1, Point p2) {
		p1.x = Math.max(p1.x, p2.x);
		p1.y = Math.max(p1.y, p2.y);
	}

	private float spanOrigin (float lower, float upper, float alignment) {
		float origin;
		if (MathUtil.equal(lower, upper, 1e-4F))
			origin = 0F;
		else
			origin = lower + alignment * (upper - lower);
		return origin;
	}

	public String toString () {
	    return new String("[lower_.x=" + lower_.x + "][lower_.y=" + lower_.y + "][upper_.x=" + upper_.x + "][upper_.y=" + upper_.y + "]");
	}
}
