package sgraphics;

// Copyright (c) 1996    Michael A. Jones

import sgraphics.layout.Glue;

/**
 * A canvas. The canvas calls allocate (Allocation) on each child
 * with the x and y positions equal to the canvas offset. The child should
 * draw itself relative to that offset.
 * <p>
 * The requirements and the allocation given to each child will be scaled
 * according to the scale factor. A child glyph should scale its
 * x and y position by the amount that its size is scaled. This behavior on
 * the part of the children will allow the canvas to scroll and zoom.
 * <p>
 * Layout glyphs will generally try to draw themselves at (0,0), relative to
 * their allocation. Therefore,
 * for a canvas to be useful, one should create a figure or composite figure
 * that can manage the scaling.
 */
public class CanvasGlyph extends PolyGlyph {
    private float yOffset_ = 0F;
    private float xOffset_ = 0F;
    private float scale_ = 1F;
    private float naturalHeight_;
    private float naturalWidth_;
    private Allocation allocation_;

    /**
     * Create a canvas of the given size that can shrink to 0 and stretch
     * forever.
     *
     * @param naturalWidth the preferred width of the canvas.
     * @param naturalHeight the preferred height of the canvas.
     */
    public CanvasGlyph (float naturalWidth, float naturalHeight) {
        naturalWidth_ = naturalWidth;
        naturalHeight_ = naturalHeight;
    }

    public Requirements requirements () {
		Requirements r = new Requirements();

		r.x.defined = true;
		r.x.natural = naturalWidth_ * scale_;
		r.x.maximum = Glue.Fil;
		r.x.minimum = 0;
		r.x.align = 0;

		r.y.defined = true;
		r.y.natural = naturalHeight_ * scale_;
		r.y.maximum = Glue.Fil;
		r.y.minimum = 0;
		r.y.align = 0;

        return r;
    }

    public void allocate (Allocation allocation) {
        allocation_ = allocation;

		BoundingSpan xSpan = allocation.allocation.span(Area.xAxis);
		BoundingSpan ySpan = allocation.allocation.span(Area.yAxis);

        int childCount = count();
        for (int pos = 0; pos < childCount; pos++) {
            Glyph child = getChild(pos);
            Requirements r = child.requirements();
            Allocation a = new Allocation();
            a.allocation = new Area(xSpan.begin - xOffset_, ySpan.begin - yOffset_, 0F, xSpan.begin + r.x.natural * scale_ - xOffset_ - 1, ySpan.begin + r.y.natural * scale_ - yOffset_ - 1, 0F);
            a.clipping = new Area(xSpan.begin - xOffset_, ySpan.begin - yOffset_, 0F, xSpan.begin + r.x.natural * scale_ - xOffset_ - 1, ySpan.begin + r.y.natural * scale_ - yOffset_ - 1, 0F);
            child.allocate(a);
        }
    }

    /**
     * Get the allocation. This used by the children of CanvasGlyph
     * so it can know the legal drawing area. (Nor for normal use.)
     */
    public Allocation getAllocation () {
        return allocation_;
    }

    /**
     * Get the scale factor.
     *
     * @return the scale factor.
     */
    public float getScale () {
        return scale_;
    }

    /**
     * Set the scale factor.
     *
     * @param the scale factor.
     */
    public void setScale (float scale) {
        scale_ = scale;
        needResize();
    }

    /**
     * Set the horizontal offset of the canvas. The offset is not scaled.
     *
     * @param offset the offset.
     */
    public void setXOffset (float offset) {
        xOffset_ = offset;
        needResize();
    }

    /**
     * Set the vertical offset of the canvas. The offset is not scaled.
     *
     * @param offset the offset.
     */
    public void setYOffset (float offset) {
        yOffset_ = offset;
        needResize();
    }

    public void draw (Painter painter) {
        // @@ Breaks other examples! Check out clipping issues on net.
        // Need whole strategy! Perhaps just get scroll bar to update
        // is enough for now.
        painter.clipRect(allocation_.clipping);
        super.draw(painter);
    }
}

