package sgraphics;

// Copyright (c) 1987-91 Stanford University
// Copyright (c) 1991-95 Silicon Graphics, Inc.
// Copyright (c) 1993-95 2001 S.A.
// Copyright (c) 1993-96 Fujitsu, Ltd.
// Copyright (c) 1994-96 Firefly Design
// Copyright (c) 1995-96 Vitria, Inc.
// Copyright (c) 1996    Michael A. Jones
// Copyright (c) 1997    Bill Petheram

import java.awt.Graphics;

/**
 * The Glyph is the smallest unit in the Sgraphics library.
 * Glyphs can be assembled in a tree and they allow
 * children to overlap, but instances can only be used once
 * in a tree.<P>
 * It is up to the user to insure there are no circular references
 * or any instance sharing!<P>
 */
public interface Glyph {
	/**
	 * Set the parent glyph. It is the responsibility of the parent
	 * to call this and pass its pointer. (Not used for normal use.)
	 *
	 * @parent parent the parent glyph.
	 */
	 public void setParent (Glyph parent);

	/**
	 * Get the parent glyph. (Not used for normal use.)
	 */
	 public Glyph getParentGlyph ();

	/**
	 * Ask for the glyph's geometry requirements. A composite glyph
	 * can store its requisition rather than compute it to speed
	 * up the system as long as its children notify the parent when
	 * their size changes.
	 *
	 * @return the requirements.
	 */
	public Requirements requirements ();

	/** Allocate the size of the region where the glyph will draw.
	 * This operation is called by the parent after it has computed
	 * the actual size its child must use. The child is responsible
	 * for determining how to deal with a size that is smaller or
	 * larger than its requirement, but should not draw outside the
	 * allocation.
	 *
	 * @param allocation the actual screen space the glyph should use to draw.
	 */
	public void allocate (Allocation allocation);

	/**
	 * Traverse the glyph tree and return all the glyphs
	 * that intersect the position of the given event.
	 * This starts at the top of a tree and proceeds downward.
	 *
	 * @param pick the traversal object.
	 */
	public void pick (Picker pick);

	/**
	 * Get the body of a glyph. (This returns null for PolyGlyph.)
	 *
	 * @return the body.
	 */
	public Glyph body ();

	/**
	 * Set the body of the glyph. (This does noththing for
	 * a PolyGlyph.)
	 *
	 * @param param body the child (body) glyph.
	 */
	 public void setBody (Glyph body);

	/**
	 * Append a glyph to the composite list. (This does nothing for
	 * a MonoGlyph.)
	 *
	 * @param glyph the glyph appended.
	 */
    public void append (Glyph glyph);

	/**
	 * Prepend a glyph to the composite list. (This does nothing for
	 * a MonoGlyph.)
	 *
	 * @param glyph the glyph appended.
	 */
    public void prepend (Glyph glyph);

	/**
	 * Replace a glyph to the composite list. (This does nothing for
	 * a MonoGlyph.) If the position is bad, nothing happens and the
	 * no error is reported.
	 *
	 * @param position the glyph to replace.
	 * @param glyph the glyph appended.
	 */
    public void replace (int position, Glyph glyph);

	/**
	 * Swap to child glyphs in the composite list.
	 *  (This does nothing for a MonoGlyph.) If a position is bad,
	 * nothing happens and the no error is reported.
	 *
	 * @param position1 the first glyph to swap.
	 * @param position2 the second glyph to swap.
	 */
    public void swap (int position1, int position2);

    /**
     * Get a child glyph;
     * @param pos the position of the child.
     * @return the child or null if the pos is too large.
     */
    public Glyph getChild (int pos);

	/**
	 * Remove all the glyphs added with append() or prepend();
	 */
	public void empty ();

	/**
	 * Get the number of child glyphs for a PolyGlyph.
	 *
	 * @return the count;
	 */
	public int count ();

	/**
	 * This is called when a glyph needs to be redrawn because it has changed
	 * internal state.
	 */
	public void needRedraw ();

	/**
	 * This is called when an area needs to be redrawn.
	 *
	 * @param area the area to redraw.
	 */
	public void needRedraw (Area area);

	/**
	 * This is called when a glyph needs to be redrawn because it has changed
	 * size.
	 */
	public void needResize ();

	/**
	 * Tell the glyph to draw itself with the given graphics.
	 *
	 * @param painter the place to draw.
	 */
	public void draw (Painter painter);

	/**
	 * Grab the events. No other viewer will receive events.
	 *
	 * @param target the glyph that is grabbing the events.
	 */
	public void grab (Viewer target);

	/**
	 * Ungrab the events.
	 */
	public void ungrab ();

	/**
	 * Get the adapter.
	 */
	public SgraphicsAdapter getSgraphicsAdapter ();
}
