package sgraphics;

// Copyright (c) 1987-91 Stanford University
// Copyright (c) 1991-95 Silicon Graphics, Inc.
// Copyright (c) 1993-95 2001 S.A.
// Copyright (c) 1993-96 Fujitsu, Ltd.
// Copyright (c) 1994-96 Firefly Design
// Copyright (c) 1995-96 Vitria, Inc.
// Copyright (c) 1996    Michael A. Jones
// Copyright (c) 1997    Bill Petheram

import java.awt.Graphics;

public abstract class GlyphImpl implements Glyph, Axis {
	private Glyph parent_ = null;
	private int size_ = 10;
	private int count_ = 0;
	private static final int increment_ = 10;
	protected Glyph children_[] = new Glyph[size_];
	protected Glyph body_ = null;


	public synchronized void setParent (Glyph parent) {
		parent_ = parent;
	}

	public synchronized Glyph getParentGlyph () {
		return parent_;
	}

	public void allocate (Allocation allocation) {}
	public void pick (Picker pick) {}

	public Glyph body () {
		return body_;
	}

	public void setBody (Glyph body) {
	    Glyph oldBody = null;
		if (body_ != null) {
	        // Set parent to null so that the GraphicsViewer
	        // can find orphans.
			body_.setParent(null);
			oldBody = body_;
		}
		body_ = body;
		if (body_ != null)
		    body_.setParent(this);
		if (oldBody != null) {
            // Get the adapter to clean up.
		    SgraphicsAdapter adapter = getSgraphicsAdapter();
		    if (adapter != null)
		        adapter.removeOrphanWidgets();
		}
    }

	public void append (Glyph glyph) {
		checkSize();
		children_[count_++] = glyph;
		glyph.setParent(this);
	}

	public void prepend (Glyph glyph) {
		checkSize();

		// Shift to the right by one.
		for (int cnt = count_; cnt > 0; cnt--)
			children_[cnt] = children_[cnt - 1];
		children_[0] = glyph;
		count_++;
		glyph.setParent(this);
	}

	public void replace (int position, Glyph glyph) {
		// Safe behavior.
		if (position >= count_)
			return;

		children_[position].setParent(null);
		children_[position] = glyph;
		glyph.setParent(this);

		SgraphicsAdapter adapter = getSgraphicsAdapter();
		if (adapter != null)
		    adapter.removeOrphanWidgets();
	}

    public void swap (int position1, int position2) {
		// Safe behavior.
		if (position1 >= count_ || position2 >= count_)
			return;

		Glyph glyph = children_[position1];
		children_[position1] = children_[position2];
		children_[position2] = glyph;
	}


   	 public Glyph getChild (int pos) {
        	if (pos >= count_)
            		return null;
        	else
            		return children_[pos];
	}

	public void empty () {
	    // Set parents to null so that the GraphicsViewer
	    // can find orphans.
	    for (int cnt = 0; cnt < count_; cnt++)
	        children_[cnt].setParent(null);

		children_ = new Glyph[size_];
		count_ = 0;

	    SgraphicsAdapter adapter = getSgraphicsAdapter();
        // Get the adapter to clean up.
        if (adapter != null)
    	    adapter.removeOrphanWidgets();
		}

	public int count () {
	    return count_;
	}

	public void grab (Viewer target) {
	    if (parent_ != null)
	        parent_.grab(target);
	}

	public void ungrab () {
	    if (parent_ != null)
	        parent_.ungrab();
	}

	/**
	 * This calls up the parent chain which leads to redrawing
	 * all the glyphs under the Viewer.
	 */
	public void needRedraw () {
		if (parent_ != null) {
			parent_.needRedraw();
			}
	}

	/**
	 * This calls up the parent chain which leads to redrawing
	 * the area.
	 */
	public void needRedraw (Area area) {
		if (parent_ != null) {
			parent_.needRedraw(area);
			}
	}

	/**
	 * This calls up the parent chain which leads to resizing and redrawing
	 * all the glyphs under the Viewer.
	 */
	public void needResize () {
		if (parent_ != null) {
			parent_.needResize();
		}
	}

	public void draw (Painter painter) {}

	public SgraphicsAdapter getSgraphicsAdapter () {
	    if (parent_ != null)
	        return parent_.getSgraphicsAdapter();
	    else
	        return null;
	}

	// Check the child list and extend it if necessary.
	private void checkSize () {
		if (count_ == size_) {
			Glyph newList[] = new Glyph[size_ + increment_];
			for (int cnt = 0; cnt < size_; cnt++)
				newList[cnt] = children_[cnt];
			size_ += increment_;
			children_ = newList;
			}
		}

	//*****************************************************************
	//                HELPER FUNCTIONS
	//*****************************************************************
	public static void requireLeadTrail(Requirement r, float naturalLead, float maxLead, float minLead, float naturalTrail, float maxTrail, float minTrail) {
		float zero = 0F;
		r.defined = true;
		naturalLead = Math.max(minLead, Math.min(maxLead, naturalLead));
		maxLead = Math.max(maxLead, naturalLead);
		minLead = Math.min(minLead, naturalLead);
		naturalTrail = Math.max(minTrail, Math.min(maxTrail, naturalTrail));
		maxTrail = Math.max(maxTrail, naturalTrail);
		minTrail = Math.min(minTrail, naturalTrail);
		r.natural = naturalLead + naturalTrail;
		if (Math.abs(naturalLead) < 0.001) {
			r.minimum = minTrail;
			r.maximum = maxTrail;
			r.align = 0F;
			}
		else if (Math.abs(naturalTrail) < 0.001) {
			r.minimum = minLead;
			r.maximum = maxLead;
			r.align = 1F;
			}
		else {
			r.minimum = r.natural * Math.max(minLead / naturalLead, minTrail / naturalTrail);
			r.maximum = r.natural * Math.min(maxLead / naturalLead, maxTrail / naturalTrail);
			if (Math.abs(r.natural) < 0.001) {
				r.align = zero;
				}
			else {
				r.align = naturalLead / r.natural;
				}
			}
	}

	public static float computeSqueeze (Requirement r, float length) {
		float f;
		float nat = r.natural;
		if (length - 0.001 > nat && r.maximum - 0.001 > nat) {
			f = (length - nat) / (r.maximum - nat);
			}
		else if (length + 0.001 < nat && r.minimum + 0.001 < nat) {
			f = (nat - length)  / (nat - r.minimum);
			}
		else {
			f = 0F;
			}
		return f;
		}

	public static Requirement requirement (Requirements r, int axis) {
		switch (axis) {
			case xAxis :
				return r.x;
			case yAxis :
				return r.y;
			default:
				return null;
			}
		}

	public static void require (Requirement r, float natural, float stretch, float shrink, float alignment) {
        r.defined = true;
        r.natural = natural;
        r.maximum = natural + stretch;
        r.minimum = natural - shrink;
        r.align = alignment;
    }

}

