package sgraphics;

// Copyright (c) 1996    Michael A. Jones

import java.awt.*;
import java.awt.image.*;

/**
 * Painter is a wrapper around awt.Graphics that uses dimensions of type float.
 */
public class Painter {
	private Graphics graphics_;
	private Sgraphics s_ = new Sgraphics();

    /**
     * Create a Painter from a Graphics.
     */
    protected Painter (Graphics graphics) {
        graphics_ = graphics;
    }

    /**
     * Get the original Graphics object. This is not normally used; however, if it
     * is neccessary to use utility classes that need the Graphics object use
     * the Sgraphics class to convert to and from resolution independent units.
     * @return the graphics object.
     */
    public Graphics getGraphics () {
        return graphics_;
    }

    /**
     * Translates the specified parameters into the origin of the graphics context. All subsequent
     * operations on this graphics context will be relative to this origin.
     * @param x the x coordinate
     * @param y the y coordinate
     */
    public void translate(float x, float y) {
        graphics_.translate(s_.convertHorizontal(x), s_.convertVertical(y));
    }

    /**
     * Gets the current color.
     * @see #setColor
     */
    public Color getColor() {
        return graphics_.getColor();
    }

    /**
     * Sets the current color to the specified color. All subsequent graphics operations
     * will use this specified color.
     * @param c the color to be set
     * @see Color
     * @see #getColor
     */
    public void setColor(Color c) {
        graphics_.setColor(c);
    }

    /**
     * Sets the paint mode to overwrite the destination with the
     * current color.
     */
    public void setPaintMode() {
        graphics_.setPaintMode();
    }

    /**
     * Sets the paint mode to alternate between the current color
     * and the new specified color.  When drawing operations are
     * performed, pixels which are the current color will be changed
     * to the specified color and vice versa.  Pixels of colors other
     * than those two colors will be changed in an unpredictable, but
     * reversible manner - if you draw the same figure twice then all
     * pixels will be restored to their original values.
     * @param c1 the second color
     */
    public void setXORMode(Color c1) {
        graphics_.setXORMode(c1);
    }

    /**
     * Gets the current font.
     * @see #setFont
     */
    public Font getFont() {
        return graphics_.getFont();
    }

    /**
     * Sets the font for all subsequent text-drawing operations.
     * @param font the specified font
     * @see Font
     * @see #getFont
     * @see #drawString
     * @see #drawBytes
     * @see #drawChars
    */
    public void setFont(Font font) {
        graphics_.setFont(font);
    }

    /**
     * Gets the current font sizes.
     * @see #getFont
     */
    public FontMetrics getFontSizes() {
	    return graphics_.getFontMetrics(graphics_.getFont());
    }

    /**
     * Gets the current font sizes for the specified font.
     * @param f the specified font
     * @see #getFont
     * @see #getFontMetrics
     */
    public FontMetrics getFontMetrics(Font f) {
	    return graphics_.getFontMetrics(f);
    }

    /**
     * Returns the area of the current clipping area.
     * @see #clipRect
     */
    public Area getClipRect() {
        Rectangle r = graphics_.getClipRect();
        return new Area (s_.convertHorizontal(r.x),
            s_.convertVertical(r.y),
            0F,
            s_.convertHorizontal(r.x + r.width - 1),
            s_.convertVertical(r.y + r.height - 1),
            0);
    }

    /**
     * Clips to a rectangle. The resulting clipping area is the
     * intersection of the current clipping area and the specified
     * area. Graphic operations have no effect outside of the
     * clipping area.
     * @param area the clip area.
     * @see #getClipRect
     */
    public void clipRect(Area area) {
        BoundingSpan xspan = area.span(Area.xAxis);
        BoundingSpan yspan = area.span(Area.yAxis);
        graphics_.clipRect(s_.convertHorizontal(xspan.begin),
            s_.convertVertical(yspan.begin),
            s_.convertHorizontal(xspan.length),
            s_.convertVertical(yspan.length));
    }

    /**
     * Copies an area of the screen.
     * @param x the x-coordinate of the source
     * @param y the y-coordinate of the source
     * @param width the width
     * @param height the height
     * @param dx the horizontal distance
     * @param dy the vertical distance
     */
    public void copyArea(float x, float y, float width, float height, float dx, float dy) {
        graphics_.copyArea(s_.convertHorizontal(x),
            s_.convertVertical(y),
            s_.convertHorizontal(width),
            s_.convertVertical(height),
            s_.convertHorizontal(dx),
            s_.convertVertical(dy));
    }

    /**
     * Draws a line between the coordinates (x1,y1) and (x2,y2). The line is drawn
     * below and to the left of the logical coordinates.
     * @param x1 the first point's x coordinate
     * @param y1 the first point's y coordinate
     * @param x2 the second point's x coordinate
     * @param y2 the second point's y coordinate
     */
    public void drawLine(float x1, float y1, float x2, float y2) {
        graphics_.drawLine(s_.convertHorizontal(x1),
            s_.convertVertical(y1),
            s_.convertHorizontal(x2),
            s_.convertVertical(y2));
    }

    /**
     * Fills the specified rectangle with the current color.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the rectangle
     * @param height the height of the rectangle
     * @see #drawRect
     * @see #clearRect
     */
    public void fillRect(float x, float y, float width, float height) {
        graphics_.fillRect(s_.convertHorizontal(x),
            s_.convertVertical(y),
            s_.convertHorizontal(width),
            s_.convertVertical(height));
    }

    /**
     * Draws the outline of the specified rectangle using the current color.
     * Use drawRect(x, y, width-1, height-1) to draw the outline inside the specified
     * rectangle.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the rectangle
     * @param height the height of the rectangle
     * @see #fillRect
     * @see #clearRect
     */
    public void drawRect(float x, float y, float width, float height) {
    	drawLine(x, y, x + width, y);
    	drawLine(x + width, y, x + width, y + height);
    	drawLine(x, y, x, y + height);
    	drawLine(x, y + height, x + width, y + height);
    }

    /**
     * Clears the specified rectangle by filling it with the current background color
     * of the current drawing surface.
     * Which drawing surface it selects depends on how the graphics context
     * was created.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the rectangle
     * @param height the height of the rectangle
     * @see #fillRect
     * @see #drawRect
     */
    public void clearRect(float x, float y, float width, float height) {
        graphics_.clearRect(s_.convertHorizontal(x),
            s_.convertVertical(y),
            s_.convertHorizontal(width),
            s_.convertVertical(height));
    }

    /**
     * Draws an outlined rounded corner rectangle using the current color.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the rectangle
     * @param height the height of the rectangle
     * @param arcWidth the horizontal diameter of the arc at the four corners
     * @param arcHeight the horizontal diameter of the arc at the four corners
     * @see #fillRoundRect
     */
    public void drawRoundRect(float x, float y, float width, float height, float arcWidth, float arcHeight) {
        graphics_.drawRoundRect(s_.convertHorizontal(x),
            s_.convertVertical(y),
            s_.convertHorizontal(width),
            s_.convertVertical(height),
            s_.convertHorizontal(arcWidth),
            s_.convertVertical(arcHeight));
    }

    /**
     * Draws a rounded rectangle filled in with the current color.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the rectangle
     * @param height the height of the rectangle
     * @param arcWidth the horizontal diameter of the arc at the four corners
     * @param arcHeight the horizontal diameter of the arc at the four corners
     * @see #drawRoundRect
     */
    public void fillRoundRect(float x, float y, float width, float height, float arcWidth, float arcHeight) {
        graphics_.fillRoundRect(s_.convertHorizontal(x),
            s_.convertVertical(y),
            s_.convertHorizontal(width),
            s_.convertVertical(height),
            s_.convertHorizontal(arcWidth),
            s_.convertVertical(arcHeight));
    }

    /**
     * Draws a highlighted 3-D rectangle.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the rectangle
     * @param height the height of the rectangle
     * @param raised a boolean that states whether the rectangle is raised or not
     */
    public void draw3DRect(float x, float y, float width, float height, boolean raised) {
	Color c = getColor();
	Color brighter = c.brighter();
	Color darker = c.darker();

	setColor(raised ? brighter : darker);
	drawLine(x, y, x, y + height);
	drawLine(x + 1, y, x + width - 1, y);
	setColor(raised ? darker : brighter);
	drawLine(x + 1, y + height, x + width, y + height);
	drawLine(x + width, y, x + width, y + height - 1);
	setColor(c);
    }

    /**
     * Paints a highlighted 3-D rectangle using the current color.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the rectangle
     * @param height the height of the rectangle
     * @param raised a boolean that states whether the rectangle is raised or not
     */
    public void fill3DRect(float x, float y, float width, float height, boolean raised) {
	Color c = getColor();
	Color brighter = c.brighter();
	Color darker = c.darker();

	if (!raised) {
	    setColor(darker);
	}
	fillRect(x+1, y+1, width-2, height-2);
	setColor(raised ? brighter : darker);
	drawLine(x, y, x, y + height - 1);
	drawLine(x + 1, y, x + width - 2, y);
	setColor(raised ? darker : brighter);
	drawLine(x + 1, y + height - 1, x + width - 1, y + height - 1);
	drawLine(x + width - 1, y, x + width - 1, y + height - 2);
	setColor(c);
    }

    /**
     * Draws an oval inside the specified rectangle using the current color.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the rectangle
     * @param height the height of the rectangle
     * @see #fillOval
     */
    public void drawOval(float x, float y, float width, float height) {
        graphics_.drawOval(s_.convertHorizontal(x),
            s_.convertVertical(y),
            s_.convertHorizontal(width),
            s_.convertVertical(height));
    }

    /**
     * Fills an oval inside the specified rectangle using the current color.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the rectangle
     * @param height the height of the rectangle
     * @see #drawOval
     */
    public void fillOval(float x, float y, float width, float height) {
        graphics_.fillOval(s_.convertHorizontal(x),
            s_.convertVertical(y),
            s_.convertHorizontal(width),
            s_.convertVertical(height));
    }

    /**
     * Draws an arc bounded by the specified rectangle starting at
     * startAngle, where 0 degrees is at the 3-o'clock position, and
     * extending for arcAngle degrees.  Positive values for arcAngle
     * indicate counter-clockwise rotations, negative values indicate
     * clockwise rotations.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the rectangle
     * @param height the height of the rectangle
     * @param startAngle the beginning angle
     * @param arcAngle the angle of the arc (relative to startAngle).
     * @see #fillArc
     */
    public void drawArc(float x, float y, float width, float height,
				 int startAngle, int arcAngle) {
	    graphics_.drawArc(s_.convertHorizontal(x),
	    s_.convertVertical(y),
	    s_.convertHorizontal(width),
	    s_.convertVertical(height),
	    startAngle, arcAngle);
    }

    /**
     * Fills an arc using the current color. This generates a pie shape.
     * The extent of the arc is the same as is described for the drawArc
     * method.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the arc
     * @param height the height of the arc
     * @param startAngle the beginning angle
     * @param arcAngle the angle of the arc (relative to startAngle).
     * @see #drawArc
     */
    public void fillArc(float x, float y, float width, float height,
				 int startAngle, int arcAngle) {
	    graphics_.fillArc(s_.convertHorizontal(x),
	    s_.convertVertical(y),
	    s_.convertHorizontal(width),
	    s_.convertVertical(height),
	    startAngle, arcAngle);
    }

    /**
     * Draws a polygon defined by an array of x points and y points.
     * @param xPoints an array of x points
     * @param yPoints an array of y points
     * @param nPoints the total number of points
     * @see #fillPolygon
     */
    public void drawPolygon(float xPoints[], float yPoints[], int nPoints) {
        int x[] = new int[xPoints.length];
        for (int cnt = 0; cnt < xPoints.length; cnt++)
            x[cnt] = s_.convertHorizontal(xPoints[cnt]);
        int y[] = new int[yPoints.length];
        for (int cnt = 0; cnt < yPoints.length; cnt++)
            y[cnt] = s_.convertHorizontal(yPoints[cnt]);
        graphics_.drawPolygon(x, y, nPoints);
    }

    /**
     * Fills a polygon with the current color using an
     * even-odd fill rule (otherwise known as an alternating rule).
     * @param xPoints an array of x points
     * @param yPoints an array of y points
     * @param nPoints the total number of points
     * @see #drawPolygon
     */
    public void fillPolygon(float xPoints[], float yPoints[], int nPoints) {
        int x[] = new int[xPoints.length];
        for (int cnt = 0; cnt < xPoints.length; cnt++)
            x[cnt] = s_.convertHorizontal(xPoints[cnt]);
        int y[] = new int[yPoints.length];
        for (int cnt = 0; cnt < yPoints.length; cnt++)
            y[cnt] = s_.convertHorizontal(yPoints[cnt]);
        graphics_.fillPolygon(x, y, nPoints);
    }

    /**
     * Draws the specified String using the current font and color.
     * The x,y position is the starting point of the baseline of the String.
     * @param str the String to be drawn
     * @param x the x coordinate
     * @param y the y coordinate
     * @see #drawChars
     * @see #drawBytes
     */
    public void drawString(String str, float x, float y) {
        graphics_.drawString(str,
	    s_.convertHorizontal(x),
	    s_.convertVertical(y));
	}


    /**
     * Draws the specified characters using the current font and color.
     * @param data the array of characters to be drawn
     * @param offset the start offset in the data
     * @param length the number of characters to be drawn
     * @param x the x coordinate
     * @param y the y coordinate
     * @see #drawString
     * @see #drawBytes
     */
    public void drawChars(char data[], int offset, int length, float x, float y) {
	    graphics_.drawChars(data, offset, length, s_.convertHorizontal(x), s_.convertVertical(y));
    }

    /**
     * Draws the specified bytes using the current font and color.
     * @param data the data to be drawn
     * @param offset the start offset in the data
     * @param length the number of bytes that are drawn
     * @param x the x coordinate
     * @param y the y coordinate
     * @see #drawString
     * @see #drawChars
     */
    public void drawBytes(byte data[], int offset, int length, float x, float y) {
	    graphics_.drawBytes(data, offset, length, s_.convertHorizontal(x), s_.convertVertical(y));
    }

    /**
     * Draws the specified image at the specified coordinate (x, y). If the image is
     * incomplete the image observer will be notified later.
     * @param img the specified image to be drawn
     * @param x the x coordinate
     * @param y the y coordinate
     * @param observer notifies if the image is complete or not
     * @see Image
     * @see ImageObserver
     */
    public boolean drawImage(Image img, float x, float y,
				      ImageObserver observer) {
	    return graphics_.drawImage(img,
	        s_.convertHorizontal(x),
	        s_.convertVertical(y),
	        observer);
	}

    /**
     * Draws the specified image inside the specified rectangle. The image is
     * scaled if necessary. If the image is incomplete the image observer will be
     * notified later.
     * @param img the specified image to be drawn
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the rectangle
     * @param height the height of the rectangle
     * @param observer notifies if the image is complete or not
     * @see Image
     * @see ImageObserver
     */
    public boolean drawImage(Image img, float x, float y,
				      float width, float height,
				      ImageObserver observer) {
	    return graphics_.drawImage(img,
	        s_.convertHorizontal(x),
	        s_.convertVertical(y),
	        s_.convertHorizontal(width),
	        s_.convertVertical(height),
	        observer);
	}

    /**
     * Draws the specified image at the specified coordinate (x, y),
     * with the given solid background Color.  If the image is
     * incomplete the image observer will be notified later.
     * @param img the specified image to be drawn
     * @param x the x coordinate
     * @param y the y coordinate
     * @param observer notifies if the image is complete or not
     * @see Image
     * @see ImageObserver
     */
    public boolean drawImage(Image img, float x, float y,
				      Color bgcolor,
				      ImageObserver observer) {
        return graphics_.drawImage(img,
            s_.convertHorizontal(x),
            s_.convertVertical(y),
            bgcolor, observer);
    }

    /**
     * Draws the specified image inside the specified rectangle,
     * with the given solid background Color. The image is
     * scaled if necessary. If the image is incomplete the image
     * observer will be notified later.
     * @param img the specified image to be drawn
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the rectangle
     * @param height the height of the rectangle
     * @param observer notifies if the image is complete or not
     * @see Image
     * @see ImageObserver
     */
    public boolean drawImage(Image img, float x, float y,
				      float width, float height,
				      Color bgcolor,
				      ImageObserver observer) {
        return graphics_.drawImage(img,
            s_.convertHorizontal(x),
            s_.convertVertical(y),
            s_.convertHorizontal(width),
            s_.convertVertical(height),
            bgcolor, observer);
    }

    /**
     * Returns a String object representing this Painters's value.
     */
    public String toString() {
	return getClass().getName() + "[font=" + getFont() + ",color=" + getColor() + "]";
    }

}
