package sgraphics;

// Copyright (c) 1996    Michael A. Jones
// Copyright (c) 1997    Bill Petheram

/**
 * Picker is used to pick the glyph tree.
 */
public class Picker {
	public static final int AREA = 1;
	public static final int POINT = 2;

	private boolean hit_ = false;
	private static final int chunkSize_ = 4;
	private Glyph[] hitList_ = new Glyph[chunkSize_];
	private int listSize_ = chunkSize_;
	private int currentPosition_ = 0;
	private Viewer viewer_ = null;

	private static final int increment_ = 10;
	private int size_ = 10;
	private int count_ = 0;
	private int index_ = 0;
	private Picker children_[] = new Picker[size_];
	private Picker next_ = null;
	private Picker prev_ = null;

	/**
	 * The type of hit. Can be AREA or POINT.
	 */
	public int hitType;

	public float x;
	public float y;
	public float width;
	public float height;

	/**
	 * Area for a DAMAGE based pick.
	 */
	public Area area;

	/**
	 * Create a picker using an event.
	 *
	 * @param event the event to use to set the x and y.
	 */
	public Picker (Viewer viewer, GraphicsEvent event) {
		x = event.X;
		y = event.Y;
		viewer_ = viewer;
		hitType = POINT;
		}

	public Picker (Picker picker, Viewer viewer) {
	    hitType = picker.hitType;
	    area = picker.area;
	    x = picker.x;
	    y = picker.y;
	    width = picker.width;
	    height = picker.height;
		viewer_ = viewer;
	}

	/**
	 * Create a picker using an area. The alignment is not used.
	 *
	 * @param a the area to use to set the x, y, length, and height.
	 */
	public Picker (Viewer viewer, Area a) {
		BoundingSpan s = a.span(Area.xAxis);
		x = s.begin;
		width = s.length;
		s = a.span(Area.yAxis);
		y = s.begin;
		height = s.length;
		hitType = AREA;
		viewer_ = viewer;
		}

	/**
	 * Call hit when the event intersects a glyph.
	 *
	 * @param g the glyph that was hit.
	 */
	public void hit (Glyph g) {
		hit_ = true;
		if (currentPosition_ == listSize_) {
			// Expand the list.
			listSize_ += chunkSize_;
			Glyph list[] = new Glyph[listSize_];
			int len = hitList_.length;
			for (int cnt = 0; cnt < len; cnt++)
				list[cnt] = hitList_[cnt];
			hitList_ = list;
			}

		// Add the hit to the list.
		hitList_[currentPosition_++] = g;
		}

	public boolean hit () {
		return hit_;
		}

	/**
	 * Get the number of hits.
	 *
	 * @return the number of hits.
	 */
	public int hits () {
		return currentPosition_;
		}

	/**
	 * Get the hit list. Do not use the length of the list to determine
	 * the number of elements. Call hits() instead. This list may be
	 * longer than the number of hits.
	 *
	 * @return the hit list.
	 */
	public Glyph[] hitList () {
		return hitList_;
		}

	public void attach (Picker pick) {
		if (next_ != null) {
			next_.attach(pick);
		}
		else {
			next_ = pick;
			next_.setPrevious(this);
		}
	}

	protected void setPrevious (Picker pick) {
	    prev_ = pick;
	}

	public void detach (Picker pick) {
		if (next_ == pick) {
			next_ = pick.getNext();
			if (next_ != null)
			    next_.setPrevious(this);
		}
		else {
			next_.detach(pick);
		}
	}

	public Picker getNext() {
		return next_;
	}

	public Picker getPrevious() {
		return prev_;
	}

	public Viewer getViewer() {
		return viewer_;
	}

}
