package sgraphics;

// Copyright (c) 1996    Michael A. Jones
// Copyright (c) 1997    Bill Petheram

import java.awt.*;
import java.lang.Math;
import java.util.Vector;
import java.util.Enumeration;
import sgraphics.widget.Widget;

/**
 * The SgraphicsAdapter connects a glyph tree to AWT. A TopViewer is inserted
 * between the top glyph and the adapter to handle events. SgraphicsAdapter
 * handles double buffering for the tree.
 */
public class SgraphicsAdapter extends Panel {
	private TopViewer topViewer_ = null;
	private Glyph topGlyph_ = null;
	private Picker damagePick_ = null;
	private Sgraphics s_ = new Sgraphics();
	private boolean canDraw_ = true;
	private boolean doubleBuffer_ = true;
	private Image buffer_ = null;
	private boolean overrideClear_ = false;
	private int width_ = -1;
	private int height_ = -1;
	private Vector eventQueue_ = new Vector();
	private boolean processingEvent_ = false;
	private boolean allocated_ = false;

	/**
	 * Create an adapter between an AWT Container and a glyph tree.
	 * By default, SgraphicsAdapter is not double buffered and update
	 * clears the background before painting.
	 *
	 * @param glyph the root of the glyph tree.
	 */
	public SgraphicsAdapter (Glyph glyph) {
		topViewer_ = new TopViewer(glyph, this);
		setLayout(null);
	}

	/**
	 *
	 * Create an adapter between an awt Component and a glyph tree.
	 * By default, SgraphicsAdapter is not double buffered and update
	 * clears the background before painting.
	 *
	 * @param glyph the root of the glyph tree.
	 * @param buffer if true then double buffer the glyph tree.
	 * @param override if true then update() does not clear the background
	 * before painting.
	 */
	public SgraphicsAdapter (Glyph glyph, boolean buffer, boolean override) {
		this(glyph);
		doubleBuffer_ = buffer;
		overrideClear_ = override;
	}

    /**
     * Get the minimum size. The minimum size is determined from the top glyph's minimum size.
     *
     * @return the minimum size.
     */
	public Dimension minimumSize () {
		Requirements r = topViewer_.requirements();
		return new Dimension(Math.max(s_.convertHorizontal(r.x.minimum), 0), Math.max(s_.convertVertical(r.y.minimum), 0));
	}

    /**
     * Get the preferred size. The preferred size is determined from the top glyph's natural size.
     *
     * @return the preferred size.
     */
	public Dimension preferredSize () {
		Requirements r = topViewer_.requirements();
		return new Dimension(s_.convertHorizontal(r.x.natural), s_.convertVertical(r.y.natural));
	}

	// This counts on the fact GraphicsEventthat resize eventually always calls reshape.
	// This could be a problem if the implementation of Component changes.
	public synchronized void reshape (int x, int y, int width, int height) {
		super.reshape(x, y, width, height);
		if (width_ != width || height_ != height) {
		    buffer_ = null;
		}
		width_ = width;
		height_ = height;

		// There is no guarentee that this is called, so this calls it in
		// case it needs to be called before allocate.
		Requirements r = topViewer_.requirements();

		Allocation a = new Allocation();
		a.allocation = new Area(0F,  0F, r.x.align, s_.convertHorizontal(width), s_.convertVertical(height), r.y.align);
		a.clipping = new Area(0F,  0F, r.x.align, s_.convertHorizontal(width), s_.convertVertical(height), r.y.align);

		topViewer_.allocate(a);
		allocated_ = true;
	}

	/**
	 * Posts an event to the top viewer and if it does not handle the event it calls the parent
	 * class which passes the event to the parent container.
	 *
	 * @param event the event.
	 * @return true if the event is handled.
	 */
	public boolean postEvent (Event event) {
	    if (!allocated_)
	        return false;
	    processingEvent_ = true;
	    boolean result = false;
	    if (!topViewer_.postEvent(new GraphicsEvent(event.target, event.when, event.id, s_.convertHorizontal(event.x), s_.convertVertical(event.y), event.key, event.modifiers, event.arg)))
	        result = super.postEvent(event);

        while (eventQueue_.size() > 0) {
			GraphicsEvent e = (GraphicsEvent) eventQueue_.elementAt(0);
            eventQueue_.removeElementAt(0);
            if (!topViewer_.postEvent(e))
                super.postEvent((Event) e);
        }

	    processingEvent_ = false;
		return result;
	}

	/**
	 * Post a new GraphicsEvent. If it is not handled, it is passed to the parent
	 * container. A decendent viewer can use this call to introduce an
	 * event to the system. If an event is already being processed by this method
	 * or postEvent(Event) the event is placed on a queue and false is returned.
	 * Queued events are processed after any current event processing is complete.
	 *
	 * @param event the new event.
	 * @return true if the event is handled.
	 */
	public boolean postEvent (GraphicsEvent event) {
	    if (!allocated_)
	        return false;

	    if (processingEvent_) {
	        eventQueue_.addElement(event);
	        return false;
	    }

	    processingEvent_ = true;
	    boolean result = false;
		if (!topViewer_.postEvent(event))
		    // The cast makes sure we get the awt postEvent().
		    result = super.postEvent((Event) event);

        while (eventQueue_.size() > 0) {
			GraphicsEvent e = (GraphicsEvent) eventQueue_.elementAt(0);
            eventQueue_.removeElementAt(0);
            if (!topViewer_.postEvent(e))
                super.postEvent((Event) e);
        }

        processingEvent_ = false;
		return result;
	}

    public void update (Graphics g) {
        checkBuffer();

        if (overrideClear_)
	            paint(g);
        else {
		    if (doubleBuffer_ && buffer_ != null) {
			    Graphics graphics = buffer_.getGraphics();
			    Color color = graphics.getColor();
			    graphics.setColor(getBackground());
			    // @@ Netscape always paints white! Bug?
			    graphics.fillRect(0, 0, width_, height_);
			    graphics.setColor(color);
			    paint(g);
			    return;
			}
            super.update(g);
        }
    }

	/**
	 * Force the buffer to copy to the screen. The purpose of this is to
	 * call disableDrawing(), draw some things, call enableDrawing(),
	 * then finally call refresh() to update the screen. If this
	 * is called when not double buffering, nothing happens.
	 */
	public void refresh () {
		if (doubleBuffer_ && buffer_ != null) {
			getGraphics().drawImage(buffer_, 0, 0, this);
		}
	}

	public void paint (Graphics g) {
        checkBuffer();

		Painter p;
		if (doubleBuffer_ && buffer_ != null)
			p = new Painter(buffer_.getGraphics());
		else
			p = new Painter(g);

	    if (canDraw_ || (doubleBuffer_ && buffer_ != null)) {
			// Draw damaged area. It is assumed that if there is damage
			// no resizing will be taking place until the this paint is complete.
			if (damagePick_ != null) {
			    while (damagePick_ != null) {
    				Glyph hitList[] = damagePick_.hitList();
    				int len = damagePick_.hits();
    				for (int cnt = 0; cnt < len; cnt++)
    					hitList[cnt].draw(p);
    				damagePick_ = damagePick_.getNext();
    			}
				damagePick_ = null;
			}
			else
			    topViewer_.draw(p);
	    }

	    if (canDraw_ && doubleBuffer_ && buffer_ != null)
		    g.drawImage(buffer_, 0, 0, this);
	}

	/**
	 * Used to change the top glyph after construction.
	 */
	public void setTopGlyph (Glyph glyph) {
	    topViewer_.setBody(glyph);
	}

    /**
     * For internal use by widgets. Not for normal use.
     */
	public void removeOrphanWidgets () {
	    Component list[] = getComponents();
	    int size = list.length;
	    for (int cnt = 0; cnt < size; cnt++) {
	        if (list[cnt] instanceof Widget && list[cnt] instanceof Glyph) {
	            Widget leafWidget = (Widget) list[cnt];
	            Glyph leafGlyph = (Glyph) list[cnt];
	            // If orphaned then.
	            if (leafGlyph.getSgraphicsAdapter() == null)
	                leafWidget.disconnected();
	        }
	    }
	}

    /**
     * Redraws all glyphs by calling repaint().
     */
	public void needRedraw () {
		repaint();
	}

    /**
     * Redraws all glyphs overlapping the area. If area is null
     * needRedraw() is called. Do not call this from the "main"
     * thread or you will get an IllegalMonitorException.
     *
     * @param area the area that needs redrawing.
     */
	public void needRedraw (Area area) {
        if (area == null) {
            needRedraw();
            return;
        }

		damagePick_ = new Picker(topViewer_, area);
		topViewer_.pick(damagePick_);
		BoundingSpan xspan = area.span(Area.xAxis);
		BoundingSpan yspan = area.span(Area.yAxis);
		float x = xspan.begin;
		float width = xspan.length;
		float y = yspan.begin;
		float height = yspan.length;

		Graphics graphics = getGraphics();
        if (graphics != null) {
		    graphics.clipRect(s_.convertHorizontal(x), s_.convertVertical(y), s_.convertHorizontal(width), s_.convertVertical(height));
		    paint(graphics);
		}
	}

    /**
     * Forces geometry calculation and redraws all glyphs.
     */
	public void needResize () {
		invalidate();
		reshape(location().x, location().y, size().width, size().height);
		repaint();
	}

	/**
	 * Gets a painter object. If you use this, you will bypass any
	 * double buffering.
	 */
	public Painter getPainter () {
	    checkBuffer();

	    Painter p;
		if (doubleBuffer_ && buffer_ != null)
			p = new Painter(buffer_.getGraphics());
		else
			p = new Painter(getGraphics());
	    return p;
	}

	public void disableDrawing () {
	    canDraw_ = false;
	}

	public void enableDrawing () {
	    canDraw_ = true;
	}

	/**
	 * Check if double buffering is on.
	 *
	 * @return true if double buffering.
	 */
	public boolean doubleBuffering () {
	    return doubleBuffer_;
	}

	/**
	 * Set the double buffer. This will not clear an existing buffer.
	 *
	 * @param enable true enables double buffering.
	 */
	public void doubleBuffering (boolean enable) {
	    doubleBuffer_ = enable;
	}

	private void checkBuffer () {
		if (doubleBuffer_ && buffer_ == null) {
			if (buffer_ == null)
					buffer_ = createImage(width_, height_);
		}

	}
}
