package sgraphics;

// Copyright (c) 1996    Michael A. Jones
// Copyright (c) 1997    Bill Petheram

import java.awt.*;
import java.lang.Math;
import java.util.Vector;
import java.util.Enumeration;
import sgraphics.widget.Widget;

/**
 * The TopViewer is topmost viewer in the viewer tree. Actually, a tree
 * does not exist, but when a picker chain is assembled by pick(...),
 * TopViewer is at the beginning of the list.
 */
public class TopViewer extends ViewerImpl {
	private SgraphicsAdapter adapter_ = null;
	private Viewer target_ = null;
	private Vector entered_ = new Vector();

    /**
     * Construct a TopViewer.
     *
     * @param glyph the top most glyph in the glyph tree that TopViewer
     * manages events for.
     * @param adapter the Adapter that TopViewer processes events for.
     */
	public TopViewer (Glyph glyph, SgraphicsAdapter adapter) {
		super(glyph);
		adapter_ = adapter;
	}

    /**
     * Handle the events. This does not do anything.
     */
	public boolean handleEvent (GraphicsEvent event) {
		return false;
	}

    /**
     * Post an AWT event into the viewer system for processing.
     *
     * @param event the event to be processed by the viewer system.
     * @return true if processed. Event grabbing does not
     * affect the return value.
     */
	public boolean postEvent (GraphicsEvent event) {

		/*
		// Certain events are not used.
		// ENTER and EXIT events.
		if (event.id == GraphicsEvent.MOUSE_ENTER ||
			event.id == GraphicsEvent.MOUSE_EXIT ||
			event.id == GraphicsEvent.ACTION_EVENT || // Warning: This is letting user events through.
			event.id == GraphicsEvent.GOT_FOCUS ||
			event.id == GraphicsEvent.LOST_FOCUS) {
			    return false;
			}
        */

		// Check to see if the events are being grabbed.
		if (target_ != null) { // Awt events.
			// I'm grabbing
			event.target = target_;
			return target_.handleEvent(event);
		}

		Picker p;
		p = new Picker(this, event);
		pick(p);
		if (p.hitType == Picker.POINT) {
		    // Picker converted the ints to floats and we save them in the event.
		    event.X = p.x;
		    event.Y = p.y;
		}

		// Check for any exit events. This is done before enter to insure that one
		// glyph is exited before another is entered. In the existing implmentation
		// there should only be one viewer in the list, but the whole list is checked
		// just in case the pick implmentation is changed in a sub-class.
		Enumeration list = entered_.elements();
		while (list.hasMoreElements()) {
		    Viewer v = (Viewer) list.nextElement();
		    boolean needsExit = true;
		    Picker tmpPick = p;
		    while (tmpPick != null) {
				if (v == tmpPick.getViewer())
					needsExit = false;
			    tmpPick = tmpPick.getNext();
			}
		    if (needsExit) {
				// Send exit event. Ignore return. Does not propagate up the picker chain!
				v.handleEvent(new GraphicsEvent(v, event.when, GraphicsEvent.MOUSE_EXIT, event.X, event.Y, event.key, event.modifiers));
				// Remove the event from the list.
				entered_.removeElement(v);
			}
		}

        Picker tmpPick = p;
		while (tmpPick != null) {
			boolean alreadyEnteredGlyph = false;
			// Check for enter.
			list = entered_.elements();
			while (list.hasMoreElements()) {
		        if ((Viewer) list.nextElement() == tmpPick.getViewer()) {
				    alreadyEnteredGlyph = true;
				    break;
				}
			}

			// Enter events are generated before the normal event.
			if (!alreadyEnteredGlyph && (tmpPick.getViewer() != this)) {
				// Send enter event. Ignore return. Does not propagate up the picker chain!
				tmpPick.getViewer().handleEvent(new GraphicsEvent(tmpPick.getViewer(), event.when, GraphicsEvent.MOUSE_ENTER, event.X, event.Y, event.key, event.modifiers));
				// Save the event on the list;
				entered_.addElement(tmpPick.getViewer());
			}
            tmpPick = tmpPick.getNext();
		}
		return processPick(p, event);
    }

	/**
	 * Grab all the events. No other viewers will receive events untill ungrab
	 * is called.
	 *
	 * @param target the glyph to deliver the events to.
	 */
	public void grab (Viewer target) {
	    // Don't allow a grab once a grab is in place.
	    if (target_ == null)
		    target_ = target;
		}

	/**
	 * Ungrab all the events.
	 */
	public void ungrab () {
		target_ = null;
		}

    /**
     * Redraws the whole glyph tree.
     */
	public void needRedraw () {
	    adapter_.needRedraw();
	}

    /**
     * Redraws all glyphs overlapping with the area.
     *
     * @param area the area that needs redrawing.
     */
	public void needRedraw (Area area) {
        adapter_.needRedraw(area);
	}

    /**
     * Forces geometry calculation and redraws the whole glyph tree.
     */
	public void needResize () {
	    adapter_.needResize();
	}

	public SgraphicsAdapter getSgraphicsAdapter () {
	    return adapter_;
	}
}

