package sgraphics;

// Copyright (c) 1996    Michael A. Jones
// Copyright (c) 1997    Bill Petheram

import java.util.Enumeration;
import java.util.Vector;
import java.awt.Canvas;

/**
 * A viewer that is used to view a tree glyphs or figures. Viewers are
 * chained by pick and processEvent(...) dispatches calls to handleEvent()
 * when there is a hit by glyphs under the viewer. Another viewer
 * qualifies as another glyph.
 * <p>
 * Note: The event MOUSE_DOWN calls grab() and the event MOUSE_UP calls
 * ungrab(), therefore, calling grab() between these events has
 * no effect. In other words, calling grab() in mouseDown(...),
 * or mouseDrag(...) is pointless and useless.
 */
public class ViewerImpl extends MonoGlyph implements Viewer {
	private Glyph parent_ = null;
	private Glyph hitList_[] = null;
	private boolean grabbed_ = false;
	private ViewerCallback callback_ = null;

    /**
     * Make a viewer of a glyph.
     *
     * @param glyph the viewed glyph.
     */
	public ViewerImpl (Glyph glyph) {
		super(glyph);
		setCallback(null);
		}

    /**
     * Make a viewer of a glyph with a callback. As a side effect the
     * ViewerCallback functions in Viewer will no longer be called.
     *
     * @param glyph the viewed glyph.
     * @param callback the callback object.
     */
    public ViewerImpl (Glyph glyph, ViewerCallback callback) {
        super(glyph);
        setCallback(callback);
    }

    /**
     * Set a callback. As a side effect the ViewerCallback functions in Viewer
     * will no longer be called.
     *
     * @param callback the callback object.
     */
    public void setCallback (ViewerCallback callback) {
        callback_ = callback;
    }

    /**
     * Remove the callback. The ViewerCallback methods in Viewer
     * will then be called.
     */
    public void removeCallback () {
        callback_ = null;
    }

	public void setParent (Glyph parent) {
		super.setParent(parent);
		parent_ = parent;
		}

    /**
     * Ask if this viewer is grabbing the events.
     *
     * @return true if grabbing.
     */
	public boolean grabbing () {
	    return grabbed_;
	}

	/**
	 * Checks for hits. Builds a chain of pickers.
	 *
	 * @param pick the picker.
	 */
    public void pick (Picker pick) {
		// Note, the hitList_ is specifically not set to null
		// anywhere so that an exit event can pickup the list.

        // Create a picker for this viewer.
        Picker p = new Picker(pick, this);

        // This will call pick on my body.
        super.pick(p);
        // If a decendent glyph or viewer was hit then by definition
        // I am hit also. Therefore, I must call hit on the picker given me.
        if (p.hit()) {
            pick.hit(this);
            // Save my hit list for later.
            hitList_ = p.hitList();
        }

        // If I am chained or was hit then attach my picker to my ancestor's picker.
        if (p.getNext() != null || p.hit())
                pick.attach(p);
    }

	/**
	 * Gets an array of decendent glyphs that were hit after the last pick.
	 * If no glyphs were hit, this returns null.
	 *
	 * @return the hit list.
	 */
	public Glyph[] getHitList () {
	    return hitList_;
	}

	/**
	 * Process a pick. Calls processPick(...) on the next viewer first.
	 * If processPick(...) on the next viewer returns false and this
	 * viewer has a hit, process the event.
	 *
	 * @param pick a picker chain with a path to all the hits.
	 * @param event the event.
	 *
	 * @param return true if the event was processed.
	 */
	public boolean processPick (Picker pick, GraphicsEvent event) {

        boolean eventProcessed = false;
        Picker next = pick.getNext();
        // Let the next viewer go first.
        if (next != null) {
            Viewer v = next.getViewer();
            eventProcessed = v.processPick(next, event);
            pick.detach(next);
        }

        // Only handle the event here if it was not handled already and
		// there was a hit.
        if (!eventProcessed && pick.hit()) {
			// We could assume this == v, but this code may move to TopViewer.
			Viewer v = pick.getViewer();
			event.target = v;
            eventProcessed = v.handleEvent(event);
		}

        // This will return true if the event is handled anywhere, i.e.
        // this viewer OR a decendent viewer.
        return eventProcessed;
    }

	/**
	 * Handles the event. Returns true if the event is handled. The default
	 * event handler calls some helper methods to make life easier
	 * on the programmer. If a callback was registered, then the helper
	 * methods of the callback object are called instead of the ones
	 * in this object.
	 *
	 * @param evt the event
	 * @return true if the event was handled.
	 *
	 * @see #mouseEnter
	 * @see #mouseExit
	 * @see #mouseMove
	 * @see #mouseDown
	 * @see #mouseDrag
	 * @see #mouseUp
	 * @see #keyDown
	 * @see #action
	 */
	public boolean handleEvent (GraphicsEvent evt) {

		 if (callback_ == null) {
			switch (evt.id) {

				case GraphicsEvent.MOUSE_ENTER:
					return mouseEnter(evt, evt.X, evt.Y);

				case GraphicsEvent.MOUSE_EXIT:
					return mouseExit(evt, evt.X, evt.Y);

				case GraphicsEvent.MOUSE_MOVE:
					return mouseMove(evt, evt.X, evt.Y);

				case GraphicsEvent.MOUSE_DOWN:
						grab(this);
						grabbed_ = true;
						return mouseDown(evt, evt.X, evt.Y);

				case GraphicsEvent.MOUSE_DRAG:
					return mouseDrag(evt, evt.X, evt.Y);

				case GraphicsEvent.MOUSE_UP:
					if (grabbed_)
						ungrab();
					grabbed_ = false;
					return mouseUp(evt, evt.X, evt.Y);

				case GraphicsEvent.KEY_PRESS:
					return keyDown(evt, evt.key);

				case GraphicsEvent.KEY_ACTION:
					return keyActionDown(evt, evt.key);

				case GraphicsEvent.KEY_RELEASE:
					return keyUp(evt, evt.key);

				case GraphicsEvent.KEY_ACTION_RELEASE:
					return keyActionUp(evt, evt.key);

				}
		}
        else {
            switch (evt.id) {

                case GraphicsEvent.MOUSE_ENTER:
                    return callback_.mouseEnter(this, evt, evt.X, evt.Y);

                case GraphicsEvent.MOUSE_EXIT:
                    return callback_.mouseExit(this, evt, evt.X, evt.Y);

                case GraphicsEvent.MOUSE_MOVE:
                    return callback_.mouseMove(this, evt, evt.X, evt.Y);

                case GraphicsEvent.MOUSE_DOWN:
                        grab(this);
                        grabbed_ = true;
                        return callback_.mouseDown(this, evt, evt.X, evt.Y);

                case GraphicsEvent.MOUSE_DRAG:
                    return callback_.mouseDrag(this, evt, evt.X, evt.Y);

                case GraphicsEvent.MOUSE_UP:
                    if (grabbed_)
                        ungrab();
					grabbed_ = false;
                    return callback_.mouseUp(this, evt, evt.X, evt.Y);

                case GraphicsEvent.KEY_PRESS:
                    return callback_.keyDown(this, evt, evt.key);

                case GraphicsEvent.KEY_ACTION:
                    return callback_.keyActionDown(this, evt, evt.key);

                case GraphicsEvent.KEY_RELEASE:
                    return callback_.keyUp(this, evt, evt.key);

                case GraphicsEvent.KEY_ACTION_RELEASE:
                    return callback_.keyActionUp(this, evt, evt.key);
            }
        }
		return false;
	}

	/**
	 * Called when the mouse is pressed.
	 * @param evt the event
	 * @param X the x coordinate
	 * @param Y the y coordinate
	 * @see #handleEvent
	 */
	public boolean mouseDown(GraphicsEvent evt, float X, float Y) {
		return false;
	}

	/**
	 * Called when the mouse is dragged (the mouse button is down).
	 * @param evt the event
	 * @param X the x coordinate
	 * @param Y the y coordinate
	 * @see #handleEvent
	 */
	public boolean mouseDrag(GraphicsEvent evt, float X, float Y) {
		return false;
	}

	/**
	 * Called when the mouse is released.
	 * @param evt the event
	 * @param X the x coordinate
	 * @param Y the y coordinate
	 * @see #handleEvent
	 */
	public boolean mouseUp(GraphicsEvent evt, float X, float Y) {
		return false;
	}

	/**
	 * Called when the mouse moves (the mouse button is up).
	 * @param evt the event
	 * @param X the x coordinate
	 * @param Y the y coordinate
	 * @see #handleEvent
	 */
	public boolean mouseMove(GraphicsEvent evt, float X, float Y) {
		return false;
	}

	/**
	 * Called when the mouse enters the component.
	 * @param evt the event
	 * @param X the x coordinate
	 * @param Y the y coordinate
	 * @see #handleEvent
	 */
	public boolean mouseEnter(GraphicsEvent evt, float X, float Y) {
		return false;
	}

	/**
	 * Called when the mouse exits the component.
	 * @param evt the event
	 * @param X the x coordinate
	 * @param Y the y coordinate
	 * @see #handleEvent
	 */
	public boolean mouseExit(GraphicsEvent evt, float X, float Y) {
		return false;
	}

	/**
	 * Called when a character is pressed.
	 * @param evt the event
	 * @param key the key that's pressed
	 * @see #handleEvent
	*/
	public boolean keyDown(GraphicsEvent evt, int key) {
		return false;
	}

	/**
	 * Called when a character is released.
	 * @param evt the event
	 * @param key the key that's released
	 * @see #handleEvent
	 */
	public boolean keyUp(GraphicsEvent evt, int key) {
		return false;
	}

	/**
	 * Called when an action character is pressed.
	 * @param evt the event
	 * @param key the key that's pressed
	 * @see #handleEvent
	*/
	public boolean keyActionDown(GraphicsEvent evt, int key) {
		return false;
	}

	/**
	 * Called when an action character is released.
	 * @param evt the event
	 * @param key the key that's released
	 * @see #handleEvent
	 */
	public boolean keyActionUp(GraphicsEvent evt, int key) {
		return false;
	}

	/**
	/**
	 * Called when an action occurs in the Component.
	 * @param evt the event
	 * @param what the action that's occuring
	 * @see #handleEvent
	 */
	public boolean action(GraphicsEvent evt, Object what) {
		return false;
	}
}
