package sgraphics.figure;

// Copyright (c) 1987-91 Stanford University
// Copyright (c) 1991-95 Silicon Graphics, Inc.
// Copyright (c) 1993-95 2001 S.A.
// Copyright (c) 1993-96 Fujitsu, Ltd.
// Copyright (c) 1994-96 Firefly Design
// Copyright (c) 1995-96 Vitria, Inc.
// Copyright (c) 1996    Michael A. Jones
// Copyright (c) 1997    Bill Petheram

import sgraphics.*;
import sgraphics.util.*;

import java.awt.Component;
import java.awt.Toolkit;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Image;
import java.awt.image.*;
import java.lang.Math;

/**
 * A text label.<p>
 *
 * Does not scale the point size if the allocation changes, nor does
 * the position scale. If the label is vertical, the alignment is zero
 * instead of the font's ascent.
 */
public class FigLabel extends Figure implements Runnable {
	public static final int HORIZONTAL = 1;
	public static final int VERTICAL = 2;

	private String text_;
	private Allocation allocation_ = null;
	private FontMetrics metrics_ = null;
    private Sgraphics s_ = new Sgraphics();
    private float align_;
    private boolean running_ = false;
    private boolean run_ = false;
    private boolean draw_ = true;
	private int direction_ = HORIZONTAL;
	private Image verticalImage_;
	private Component adapter_;

	public FigLabel (String text) {
		text_ = text;
	}

	public FigLabel (String text, boolean run, boolean draw) {
		text_ = text;
		run_ = run;
		draw_ = draw;
	}

	public FigLabel (String text, boolean draw, int direction) {
		text_ = text;
		draw_ = draw;
		direction_ = direction;
	}

	public void run () {
	    while(running_) {
	        SgraphicsAdapter adapter = getSgraphicsAdapter();
	        if (adapter != null) {
	            draw(adapter.getPainter());
	            // In case we are double buffered.
	            adapter.refresh();
	        } else {
				running_ = false;
	            Thread t = Thread.currentThread();
	            t.stop();
	        }
	        try {
	            Thread.currentThread().sleep(500);
	        } catch (InterruptedException e) {
	        }
	        FigureStyle style = getStyle();
	        Color c = style.foreground;
	        style.foreground = style.background;
	        style.background = c;
	        setStyle(style);
	    }
	}

	/**
	 * Return requirements based on the text.
	 *
	 * @return the requirements.
	 */
	public Requirements requirements () {
		Requirements r = new Requirements();

		if (metrics_ != null) {
			float width = s_.convertHorizontal(metrics_.stringWidth(text_));
			float height = s_.convertVertical(metrics_.getHeight());
			align_ = s_.convertVertical(metrics_.getAscent()) / height;

			if (direction_ == VERTICAL && verticalImage_ == null)
				makeVerticalImage();

			if (direction_ == VERTICAL) {
				r.y.defined = true;
				r.y.natural = width;
				r.y.maximum = width;
				r.y.minimum = width;
				r.y.align = 0;

				r.x.defined = true;
				r.x.natural = height;
				r.x.maximum = height;
				r.x.minimum = height;
				r.x.align = 0;
			}
			else {
				r.x.defined = true;
				r.x.natural = width;
				r.x.maximum = width;
				r.x.minimum = width;
				r.x.align = 0;

				r.y.defined = true;
				r.y.natural = height;
				r.y.maximum = height;
				r.y.minimum = height;
				r.y.align = align_;
			}
		}
		return r;
	}

	public void setStyle (FigureStyle style) {
		// Cause the image to be discarded.
		if (direction_ == VERTICAL)
			verticalImage_ = null;

		if (style.font != null) {
			Toolkit kit = Toolkit.getDefaultToolkit();
			metrics_ = kit.getFontMetrics(style.font);
			}
		super.setStyle(style);
		}

	public synchronized void allocate (Allocation allocation) {
		// If the allocation does not exist then set it.
		// If there is a allocation, then merge in the new one.
		allocation_ = allocation;
		}

	public synchronized void draw (Painter painter) {
	    if (!draw_)
	        return;
		// Dont draw if there is no allocation.
		if (allocation_ != null) {
		    if (run_ && !running_) {
				running_ = true;
		        Thread thread = new Thread(this);
		        thread.start();
		    }

			// Get info on where to draw.
			BoundingSpan xSpan = allocation_.allocation.span(Area.xAxis);
			BoundingSpan ySpan = allocation_.allocation.span(Area.yAxis);

			if (direction_ == VERTICAL && verticalImage_ != null) {
				painter.drawImage(verticalImage_, xSpan.begin, ySpan.begin, adapter_);
			}
			else {

				FigureStyle style = getStyle();
				Color color = painter.getColor();
				if (style != null) {
					if (style.background != null) {
						painter.setColor(style.background);
						painter.fillRect(xSpan.begin, ySpan.begin, xSpan.length, ySpan.length);
					}
					else
					    painter.clearRect(xSpan.begin, ySpan.begin, xSpan.length, ySpan.length);
					if (style.font != null)
						painter.setFont(style.font);
					if (style.foreground != null)
						painter.setColor(style.foreground);
				    else
				        painter.setColor(color);
				}

				// This speeds up X windows by sending less bytes.
				byte data[] = new byte[text_.length()];
				text_.getBytes(0, text_.length(), data, 0);
				painter.drawBytes(data, 0, text_.length(), xSpan.begin, ySpan.begin + (align_ * ySpan.length));
			}
		}
	}

	public void pick (Picker pick) {
		if (allocation_ == null)
			return;
		BoundingSpan xSpan = allocation_.allocation.span(Area.xAxis);
		BoundingSpan ySpan = allocation_.allocation.span(Area.yAxis);
		float x1 = xSpan.begin;
		float y1 = ySpan.begin;
		float x2 = xSpan.end;
		float y2 = ySpan.end;

		if (pick.hitType == Picker.POINT) {
			// @@ This is converting ints to floats.
			float x = pick.x;
			float y = pick.y;

			if (x >= x1 && x <= x2 && y >= y1 && y <= y2) {
			//System.out.println("hit a fig label");
				pick.hit(this);
				}
			}
		else if (pick.hitType == Picker.AREA) {
			// Check all four corners to see if any corner is in my area.
			// Top left.
			if (pick.x >= x1 && pick.x <= x2 && pick.y >= y1 && pick.y <= y2) {
				pick.hit(this);
				return;
				}
			// Top right.
			if (pick.x + pick.width - 1 >= x1 && pick.x + pick.width - 1 <= x2 && pick.y >= y1 && pick.y <= y2) {
				pick.hit(this);
				return;
				}
			// Bottom left.
			if (pick.x >= x1 && pick.x <= x2 && pick.y + pick.height - 1 >= y1 && pick.y + pick.height - 1 <= y2) {
				pick.hit(this);
				return;
				}
			// Bottom right.
			if (pick.x + pick.width - 1 >= x1 && pick.x + pick.width - 1 <= x2 && pick.y + pick.height - 1 >= y1 && pick.y + pick.height - 1 <= y2) {
				pick.hit(this);
				return;
				}
			}
		}

	public synchronized void needRedraw () {
	    if (allocation_ != null)
	        needRedraw(allocation_.allocation);
	    else
	        super.needRedraw();
		}

    /**
     * This will return the character position that would be hit for the
	 * given event coordinates. For horizontal labels, only x is used.
	 * For vertical labels, only y is used. Coordinates are any
	 * valid event coordinates, but if the coordinates are outside
	 * of the allocation this will return 0.
     *
     * @param x the x coordinate.
     * @param y the y coordinate.
     * @return the character position hit starting at 0. If nothing
	 * is hit, then zero is returned.
     */
	public int position (float x, float y) {
		int len = text_.length();
		FigureStyle style = getStyle();
		if (style != null && style.font != null) {
			// Get a byte array to speed things up.
			byte b[] = new byte[len];
			text_.getBytes(0, len, b, 0);

			int sizes[] = metrics_.getWidths();
			// Start at the edge of the allocation.
			BoundingSpan span;
			if (direction_ == VERTICAL)
				span = allocation_.allocation.span(Area.yAxis);
			else
				span = allocation_.allocation.span(Area.xAxis);
			float totalSize = span.origin;

			// Go through the list to see which char was hit.
			for (int cnt = 0; cnt < len; cnt++) {
				// Index b by the count and add the size of the char to totalSize;
				totalSize += s_.convertHorizontal(sizes[b[cnt]]);
				// Check if the totalSize overlaps event.X.
				if (direction_ == VERTICAL) {
					if (y < totalSize) {
						return len - cnt - 1;
					}
				}
				else {
					if (x < totalSize) {
						return cnt;
					}
				}
			}
		}
	return 0;
	}

	public int length () {
		return text_.length();
	}

	private void makeVerticalImage () {

		if (metrics_ == null)
			return;

		int width = metrics_.stringWidth(text_);
		int height = metrics_.getHeight();

		adapter_ = (Component) getSgraphicsAdapter();

		Image image = adapter_.createImage(width, height);
		if (image == null)
			return;
		Graphics g = image.getGraphics();
       	FigureStyle style = getStyle();
		if (style.font != null)
			g.setFont(style.font);

	    Color color = g.getColor();
		if (style.background != null) {
			g.setColor(style.background);
		    g.fillRect(0, 0, width, height);
		}

		if (style.foreground != null)
			g.setColor(style.foreground);
	    else
	        g.setColor(color);

		byte data[] = new byte[text_.length()];
		text_.getBytes(0, text_.length(), data, 0);
		g.drawBytes(data, 0, text_.length(), 0, metrics_.getAscent());

		int pixmap[] = new int[width * height];

		PixelGrabber grabber = new PixelGrabber(image, 0, 0, width, height, pixmap, 0, width);
		try {
			grabber.grabPixels(2000);
		}
		catch (InterruptedException e) {
			return;
		}

		int newPixmap[] = new int[width * height];

		for (int row = 0; row < height; row++)
			for (int col = 0; col < width; col++)
				newPixmap[(width - col - 1) * height + row] = pixmap[row * width + col];

		verticalImage_ = adapter_.createImage(new MemoryImageSource(height, width, newPixmap, 0, height));
	}
}
