package sgraphics.figure;

// Copyright (c) 1987-91 Stanford University
// Copyright (c) 1991-95 Silicon Graphics, Inc.
// Copyright (c) 1993-95 2001 S.A.
// Copyright (c) 1993-96 Fujitsu, Ltd.
// Copyright (c) 1994-96 Firefly Design
// Copyright (c) 1995-96 Vitria, Inc.
// Copyright (c) 1996    Michael A. Jones

import sgraphics.*;
import sgraphics.util.*;

import java.awt.Graphics;
import java.awt.Color;
import java.lang.Math;

/**
 * A rectangle.
 * <p>
 * Fill mode only works if the brushSize is 1. Clear mode calls clearRect for the
 * area.
 * <p>
 * Widths and heights are measured by counting the number of pixels between
 * the two lines plus 2 pixels for the lines. For x1 = 2 and x2 = 5, the width
 * is 4.
 * <p> The Rectangle does not stretch.
 */
class FigRectangle extends Figure {
	private Point p1_ = null;
	private Point p2_  = null;
	private Allocation allocation_ = null;

	public FigRectangle (Point p1, Point p2) {
		p1_ = p1;
		p2_ = p2;
		// Make sure p2 > p1
		if (p1_.x > p2_.x) {
			float t = p1_.x;
			p1_.x = p2_.x;
			p2_.x = t;
			}
		if (p1_.y > p2_.y) {
			float t = p1_.y;
			p1_.y = p2_.y;
			p2_.y = t;
			}
		}

	/**
	 * Return requirements based on the points of the rectangle. Requirements
	 * are calculated by subtracting the x and y points. The requirement is
	 * given for the rectangle without considering where the rectange is in
	 * the coordinate space.
	 */
	public Requirements requirements () {

		FigureStyle style = getStyle();
		if (style.brushSize < 1)
			style.brushSize = 1;

		Requirements r = new Requirements();
		r.x.defined = true;
		r.x.natural = p2_.x - p1_.x + style.brushSize;
		r.x.maximum = r.x.minimum = r.x.natural;
		r.x.align = 0;

		r.y.defined = true;
		r.y.natural = p2_.y - p1_.y + style.brushSize;
		r.y.maximum = r.y.minimum = r.y.natural;
		r.y.align = 0;

		return r;
		}

	public void allocate (Allocation allocation) {
		// If the allocation does not exist then set it.
		allocation_ = allocation;
		}

// Will need to add cliping, checking for allocation if too small, etc.
	public void draw (Painter painter) {
		// Dont draw if there is no allocation.
		if (allocation_ != null) {
			// Get info on where to draw.
			// @@ Should also set the clipping regions later.
    		BoundingSpan xSpan = allocation_.allocation.span(Area.xAxis);
    		BoundingSpan ySpan = allocation_.allocation.span(Area.yAxis);

            float x = xSpan.begin;
            float y = ySpan.begin;
            float width = xSpan.length;
            float height = ySpan.length;

			FigureStyle style = getStyle();
			if (style.foreground != null) {
				painter.setColor(style.foreground);
				}

			if (style.brushSize > 1F)
				GraphicsUtil.drawRect(painter, x +
				style.brushSize/2F, y + style.brushSize/2F, width - 1F - style.brushSize, height - 1F - style.brushSize, style.brushSize);
			else
				switch (getMode()) {
					case Figure.Stroke :
						painter.drawRect(x, y, width-1F, height-1F);
						break;
					case Figure.FillStroke :
					case Figure.Fill :
						painter.fillRect(x, y, width, height);
						break;
					case Figure.Clear :
						painter.clearRect(x, y, width, height);
						break;
					}
			}
		}

	public void pick (Picker pick) {
		if (allocation_ == null)
			return;

		BoundingSpan xSpan = allocation_.allocation.span(Area.xAxis);
		BoundingSpan ySpan = allocation_.allocation.span(Area.yAxis);

        float x1 = xSpan.begin;
        float y1 = ySpan.begin;
        float x2 = xSpan.end;
        float y2 = ySpan.end;

		if (pick.hitType == Picker.POINT) {
			float x = pick.x;
			float y = pick.y;

			if (x >= x1 && x <= x2 && y >= y1 && y <= y2) {
				pick.hit(this);
				}
			}
		else if (pick.hitType == Picker.AREA) {
			// Check all four corners to see if any corner is in my area.
			// Top left.
			if (pick.x >= x1 && pick.x <= x2 && pick.y >= y1 && pick.y <= y2) {
				pick.hit(this);
				return;
				}
			// Top right.
			if (pick.x + pick.width - 1 >= x1 && pick.x + pick.width - 1 <= x2 && pick.y >= y1 && pick.y <= y2) {
				pick.hit(this);
				return;
				}
			// Bottom left.
			if (pick.x >= x1 && pick.x <= x2 && pick.y + pick.height - 1 >= y1 && pick.y + pick.height - 1 <= y2) {
				pick.hit(this);
				return;
				}
			// Bottom right.
			if (pick.x + pick.width - 1 >= x1 && pick.x + pick.width - 1 <= x2 && pick.y + pick.height - 1 >= y1 && pick.y + pick.height - 1 <= y2) {
				pick.hit(this);
				return;
				}
			}
		}

	public void needRedraw () {
	    if (allocation_ != null)
	        needRedraw(allocation_.allocation);
	    else
	        super.needRedraw();
	}
}
