package sgraphics.figure;

// Copyright (c) 1987-91 Stanford University
// Copyright (c) 1991-95 Silicon Graphics, Inc.
// Copyright (c) 1993-95 2001 S.A.
// Copyright (c) 1993-96 Fujitsu, Ltd.
// Copyright (c) 1994-96 Firefly Design
// Copyright (c) 1995-96 Vitria, Inc.
// Copyright (c) 1996    Michael A. Jones

import sgraphics.*;
import java.awt.Image;

/**
 * A kit for figures.
 */
public class FigureKit {

	/**
	 * Get a rectangle.
	 *
	 * @param mode the mode of the figure. FigureMode.Stroke, FigureMode.Fill, or FigureMode.FillStroke.
	 * @param s the style of the figure.
	 * @param x1 the upper left x position.
	 * @param y1 the upper left y position.
	 * @param x2 the lower left x position.
	 * @param y2 the lower left y position.
	 * @return the rectangle.
	 * @see FigureMode
	 * @see FigureStyle
	 */
	public static Figure rectangle (int mode, FigureStyle s, float x1, float y1, float x2, float y2) {
		Figure f = new FigRectangle(new Point(x1, y1), new Point(x2, y2));
		f.setMode(mode);
		f.setStyle(s);
		return f;
		}

	/**
	 * Get a oval.
	 *
	 * @param mode the mode of the figure. FigureMode.Stroke, FigureMode.Fill, or FigureMode.FillStroke.
	 * @param s the style of the figure.
	 * @param x1 the upper left x position.
	 * @param y1 the upper left y position.
	 * @param x2 the lower left x position.
	 * @param y2 the lower left y position.
	 * @return the oval.
	 * @see FigureMode
	 * @see FigureStyle
	 */
	public static Figure oval (int mode, FigureStyle s, float x1, float y1, float x2, float y2) {
		Figure f = new FigOval(new Point(x1, y1), new Point(x2, y2));
		f.setMode(mode);
		f.setStyle(s);
		return f;
		}
	/**
	 * Get a circle.
	 *
	 * @param mode the mode of the figure. FigureMode.Stroke, FigureMode.Fill, or FigureMode.FillStroke.
	 * @param s the style of the figure.
	 * @param x the upper left x position.
	 * @param y the upper left y position.
	 * @param d the diameter.
	 * @return the circle.
	 * @see FigureMode
	 * @see FigureStyle
	 */
	public static Figure circle (int mode, FigureStyle s, float x, float y, float d) {
		Figure f = new FigOval(new Point(x, y), new Point(x + d - 1, y + d - 1));
		f.setMode(mode);
		f.setStyle(s);
		return f;
		}

	/**
	 * Get a line.
	 *
	 * @param s the style of the figure.
	 * @param x1 the first x position.
	 * @param y1 the first y position.
	 * @param x2 the second x position.
	 * @param y2 the second y position.
	 * @return the line.
	 * @see FigureStyle
	 */
	public static Figure line (FigureStyle s, float x1, float y1, float x2, float y2) {
		Figure f = new FigLine(new Point(x1, y1), new Point(x2, y2));
		f.setStyle(s);
		return f;
		}

	/**
	 * Get a label. The event generated is modified to reflect the
	 * character position in the label. (0, 0) is the first character.
	 * (1, 0) is the second character, etc.
	 * <p>
	 * A label is not scaleable, nor is its position. Therefore, it is
	 * of no use on a CanvasGlyph.
	 * <p>
	 * Only chars 0-255 are supported.
	 *
	 * @param s the style of the figure.
	 * @param text the string of the label.
	 * @return the label.
	 * @see FigureStyle
	 */
	public static Figure label (FigureStyle s, String text) {
		Figure f = new FigLabel(text);
		f.setStyle(s);
		return f;
		}

	/**
	 * Get a vertical label. The event generated is modified to reflect the
	 * character position in the label. (0, 0) is the first character.
	 * (1, 0) is the second character, etc.
	 * <p>
	 * A label is not scaleable, nor is its position. Therefore, it is
	 * of no use on a CanvasGlyph.
	 * <p>
	 * Only chars 0-255 are supported.
	 *
	 * @param s the style of the figure.
	 * @param text the string of the label.
	 * @return the label.
	 * @see FigureStyle
	 */
	public static Figure verticalLabel (FigureStyle s, String text) {
		Figure f = new FigLabel(text, true, FigLabel.VERTICAL);
		f.setStyle(s);
		return f;
		}
	/**
	 * Get a blinking label.
	 *
	 * @param s the style of the figure.
	 * @param text the string of the label.
	 * @return the label.
	 * @see FigureStyle
	 */
	public static Figure blinkingLabel (FigureStyle s, String text) {
		Figure f = new FigLabel(text, true, true);
		f.setStyle(s);
		return f;
		}

	/**
	 * Get a hidden label. A hidden label takes up space, but does not draw it self.
	 * Picking is still enabled.
	 *
	 * @param s the style of the figure.
	 * @param text the string of the label.
	 * @return the label.
	 * @see FigureStyle
	 */
	public static Figure hiddenLabel (FigureStyle s, String text) {
		Figure f = new FigLabel(text, false, false);
		f.setStyle(s);
		return f;
		}


    /**
     * Get an image figure. The image will not stretch or shrink. The image will give a
     * requirement of 0 until the image is loaded. The figure will observe the image
     * and call needRedraw() when the image is loaded. Therefore, large images
     * may not align properly and then suddenly they will be redrawn when they are
     * fully laoded.
     *
     * @param image the image to use.
     */
    public static Figure image (Image image) {
            return new FigImage(image);
    }

    /**
     * Get an image figure. The image will not stretch or shrink.
     *
     * @param image the image to use.
     * @param width the width of the image.
     * @param height the height of the image.
     */
    public static Figure image (Image image, float width, float height) {
            return new FigImage(image, width, height);
    }

	/**
	 * Get an insert bar. The font of the FigureStyle should be set so that
	 * the bar can calculate its alignment.
	 *
	 * @param s the style of the figure.
	 * @return the bar.
	 * @see FigureStyle
	 */
	public static Figure insertBar (FigureStyle s) {
		Figure f = new FigInsertBar();
		f.setStyle(s);
		return f;
		}
}
