package sgraphics.figure;

// Copyright (c) 1996    Michael A. Jones
// Copyright (c) 1997    Bill Petheram

import sgraphics.*;
import sgraphics.util.*;

import java.awt.Graphics;
import java.awt.Color;
import java.lang.Math;

/**
 * Make any glyph or figure usable on a canvas. This will take care
 * of scaling and scrolling. The child glyph or figure must be able
 * to change its size via the allocation(Allocation) call in order for
 * scaling to work.
 * <p>
 * Note: FigureViewer wont work if double buffering is turned off in SgraphicsAdapter.
 */
public class FigureViewer extends ViewerImpl {
	private Point p_ = null;

	private boolean drag_ = false;
	private float baseX_;
	private float baseY_;
	private float dragX_;
	private float dragY_;
	private int dragCount_;

	private Color xOrColor_ = Color.white;

	private Allocation allocation_;

	private boolean canDraw_ = true;
	private SgraphicsAdapter adapter_;
	private Painter painter_;

    /**
     * @param glyph the adapted glyph.
     * @param p the point where the upper left corner of the figure will be placed.
     */
	public FigureViewer (Glyph glyph, Point p) {
	    super(glyph);
		p_ = p;
		}

    public boolean mouseDrag (GraphicsEvent event, float X, float Y) {
	    if (drag_) {
	        // Undraw the old line.
            draw(painter_);

	        // Move then draw the new line.
	        p_ = new Point(baseX_ + (event.X - dragX_) / xScale(), baseY_ + (event.Y - dragY_) / yScale());
            allocate(allocation_);
            draw(painter_);
            adapter_.refresh();
	        return true;
	    }
	    else
	        return false;
    }

	public boolean mouseDown(GraphicsEvent event, float X, float Y) {
	        grab(this);

	        adapter_ = getSgraphicsAdapter();
            painter_ = adapter_.getPainter();
            // We will manually control the buffer.
	        adapter_.disableDrawing();

	        // Remove non XOR image first.
	        canDraw_ = false;
	        // You might be tempted to call needRedraw(), but
	        // it will cause flicker and there is nothing
	        // you can do about it (JDK 1.02). This is because
	        // needRedraw() calls repaint().
	        adapter_.update(adapter_.getGraphics());
	        canDraw_ = true;

	        // Draw with XOR.
	        // @@ Netscape wont do XOR on a buffer! Bug?
            painter_.setXORMode(xOrColor_);
            draw(painter_);
            adapter_.refresh();

	        drag_ = true;
	        baseX_ = p_.x;
	        baseY_ = p_.y;
	        dragX_ = event.X;
	        dragY_ = event.Y;
	        return true;
		}

	public boolean mouseUp(GraphicsEvent event, float X, float Y) {
	        ungrab();
            painter_.setPaintMode();
            adapter_.enableDrawing();
	        needRedraw();
	        drag_ = false;
	        return true;
		}

    public Requirements requirements () {
        return body().requirements();
    }

	public void allocate (Allocation allocation) {
	    allocation_ = allocation;
		body().allocate(scaleAllocation());
		}

    public void pick (Picker pick) {
		super.pick(pick);
	}

	public void draw (Painter painter) {
	    if (canDraw_)
	        super.draw(painter);
	}

	public void setStyle (FigureStyle style) {
	    if (body() instanceof Figure)
	        ((Figure)body()).setStyle(style);
		}

	public FigureStyle getStyle () {
	    if (body() instanceof Figure)
		    return ((Figure)body()).getStyle();
		return null;
		}

	public void setMode (int mode) {
	    if (body() instanceof Figure)
	        ((Figure)body()).setMode(mode);
		}

	public int getMode () {
	    if (body() instanceof Figure)
	        return ((Figure)body()).getMode();
	    return -1;
		}


	private Allocation scaleAllocation () {
		BoundingSpan xSpan = allocation_.allocation.span(Area.xAxis);
		BoundingSpan ySpan = allocation_.allocation.span(Area.yAxis);
		Requirements r = requirements();
		float xLen = r.x.natural;
		float yLen = r.y.natural;
		float xAdjust = xScale();
		float yAdjust = yScale();

		float x1 = xSpan.begin + p_.x * xAdjust;
		float y1 = ySpan.begin + p_.y * yAdjust;
		float x2 = x1 + xLen * xAdjust;
		float y2 = y1 + yLen * yAdjust;

		Allocation a = new Allocation();
		a.allocation = new Area(x1, y1, 0F, x2, y2, 0F);
		//a.clipping = new Area(x1 - 1, y1 - 1, 0F, x2 + 1, y2 + 1, 0F);
		return a;
	}

	private float xScale () {
		BoundingSpan span = allocation_.allocation.span(Area.xAxis);
		Requirements r = requirements();
		return ((span.length - r.x.natural) / r.x.natural) + 1F;
	}

	private float yScale () {
		BoundingSpan span = allocation_.allocation.span(Area.yAxis);
		Requirements r = requirements();
		return ((span.length - r.y.natural) / r.y.natural) + 1F;
	}
}


