package sgraphics.layout;

// Copyright (c) 1987-91 Stanford University
// Copyright (c) 1991-95 Silicon Graphics, Inc.
// Copyright (c) 1993-95 2001 S.A.
// Copyright (c) 1993-96 Fujitsu, Ltd.
// Copyright (c) 1994-96 Firefly Design
// Copyright (c) 1995-96 Vitria, Inc.
// Copyright (c) 1996    Michael A. Jones


import sgraphics.*;
import sgraphics.util.*;

/**
 * A layout class that aligns on an axis.
 */
class AlignLayout extends LayoutManager implements Axis {
	private int axis_;

	/**
	 * @param axis the axis to file along.
	 */
	public AlignLayout (int axis) {
		axis_ = axis;
		}

    /**
     * Returns a requirement that is large enough for all glyphs and has an alignment
     * that marks where all the glyphs should be aligned.
     */
	public void requirements (Glyph list[], Requirements requirements) {
		// Find out how many Glyphs there are;
		int listSize = list.length;
		int listLen = 0;
		while (listLen < listSize) {
			if (list[listLen] == null)
				break;
			listLen++;
			}

		Requirement r = null;
		float naturalLead = 0F;
		float naturalTrail = 0F;
		float minLead = -Glue.Fil - 1F;
		float maxLead = Glue.Fil + 1F;
		float minTrail = -Glue.Fil - 1F;
		float maxTrail = Glue.Fil + 1F;

		for (int cnt = 0; cnt < listLen; cnt++) {
			r = GlyphImpl.requirement(list[cnt].requirements(), axis_);
			if (r.defined) {
				float rNat = r.natural;
				float rMax = r.maximum;
				float rMin = r.minimum;
				float rAlign = r.align;
				float rInvAlign = 1F - rAlign;
				naturalLead = Math.max(naturalLead, rNat * rAlign);
				maxLead = Math.min(maxLead, rMax * rAlign);
				minLead = Math.max(minLead, rMin * rAlign);
				naturalTrail = Math.max(naturalTrail, rNat * rInvAlign);
				maxTrail = Math.min(maxTrail, rMax * rInvAlign);
				minTrail = Math.max(minTrail, rMin * rInvAlign);
				}
			}

		r = GlyphImpl.requirement(requirements, axis_);
		GlyphImpl.requireLeadTrail(r, naturalLead, maxLead, minLead, naturalTrail, maxTrail, minTrail);
		}

    /**
     * Will allocate the maximum space for each clild. It does this by stretching the
     * child as far as it can within the boundries of the allocation while keeping
     * the child aligned properly.
     */
	public void allocate (Glyph list[], Allocation allocation) {
		Allocation a[] = calculateAllocations(list, allocation, axis_);
		int length = a.length;
		for (int cnt = 0; cnt < length; cnt++) {
			list[cnt].allocate(a[cnt]);
			}
		}

	public static Allocation[] calculateAllocations (Glyph list[], Allocation allocation, int axis) {
		int listSize = list.length;
		int listLen = 0;
		while (listLen < listSize) {
			if (list[listLen] == null)
				break;
			listLen++;
			}

		Requirement r = null;
		Area a = allocation.allocation;
		BoundingSpan span = a.span(axis);
		Allocation childAllocations[] = new Allocation[listLen];
		for (int cnt = 0; cnt < listLen; cnt++) {
			r = GlyphImpl.requirement(list[cnt].requirements(), axis);
			float x1 = 0;
			float x2 = 0;
			float xAlign = 0;
			float y1 = 0;
			float y2 = 0;
			float yAlign = 0;
			childAllocations[cnt] = new Allocation();
			if (r.defined) {
				float length = Math.max(Math.min(span.length, r.maximum), r.minimum);
                if (Math.abs(r.align) < 0.001)
                    length *= (1 - span.align);
                else if (r.align > 0.999 && r.align < 1.001)
                    length *= span.align;
                else
                    length *= Math.min(span.align/r.align, (1 - span.align)/(1 - r.align));
                length = Math.min(length, r.maximum);
                length = Math.max(length, r.minimum);
				switch (axis) {
					case xAxis :
						x1 = span.origin - (r.align * (length - 1));
						x2 = x1 + length - 1;
						xAlign = r.align;
	            		childAllocations[cnt].allocation = new Area(x1, y1, xAlign, true, x2, y2, yAlign, false);
	            		childAllocations[cnt].clipping = new Area(x1, y1, xAlign, true, x2, y2, yAlign, false);
						break;
					case yAxis :
						y1 = span.origin - (r.align * (length - 1));
						y2 = y1 + length - 1;
						yAlign = r.align;
	            		childAllocations[cnt].allocation = new Area(x1, y1, xAlign, false, x2, y2, yAlign, true);
	            		childAllocations[cnt].clipping = new Area(x1, y1, xAlign, false, x2, y2, yAlign, true);
						break;
					}
				}
			}
		return childAllocations;
		}

	public Allocation[] allocations (Glyph list[], Allocation allocation) {
		return calculateAllocations(list, allocation, axis_);
		}
}
