package sgraphics.layout;

// Copyright (c) 1987-91 Stanford University
// Copyright (c) 1991-95 Silicon Graphics, Inc.
// Copyright (c) 1993-95 2001 S.A.
// Copyright (c) 1993-96 Fujitsu, Ltd.
// Copyright (c) 1994-96 Firefly Design
// Copyright (c) 1995-96 Vitria, Inc.
// Copyright (c) 1996    Michael A. Jones

import sgraphics.*;

/**
 * LayoutKit creates architectural components.
 *
 */
public class LayoutKit {

	/**
	 * Position the glyph using the given alignments.
	 *
	 * @param g the glyph to align
	 * @param xAlign the alignement in the x axis
	 * @param yAlign the alignement in the y axis
	 * @return the glyph.
	 */
	public static Glyph align (Glyph g, float xAlign, float yAlign) {
		return new Placement(g, new SuperposeLayout(new CenterLayout(CenterLayout.xAxis, xAlign), new CenterLayout(CenterLayout.yAxis, yAlign)));
		//return new Placement(
			//new Placement(g, new CenterLayout(CenterLayout.xAxis, xAlign)),
			//new CenterLayout(CenterLayout.yAxis, yAlign));
		}

	/**
	 * Position the glyph by aligning along an axis.
	 *
	 * @param g the glyph to align
	 * @param axis the axis to align on
	 * @param align the alignment
	 * @return the glyph.
	 */
	public static Glyph alignAxis (Glyph g, int axis, float align) {
		return new Placement(g, new CenterLayout(axis, align));
		}

	/**
	 * Position the glyph by aligning along the x axis.
	 *
	 * @param g the glyph to align
	 * @param align the alignment
	 * @return the glyph.
	 */
	public static Glyph halign (Glyph g, float align) {
		return alignAxis(g, CenterLayout.xAxis, align);
		}

	/**
	 * Position the glyph by aligning along the y axis.
	 *
	 * @param g the glyph to align
	 * @param align the alignment
	 * @return the glyph.
	 */
	public static Glyph valign (Glyph g, float align) {
		return alignAxis(g, CenterLayout.yAxis, align);
		}

	/**
	 * Fix the size of a glyph along its axis.
	 *
	 * @param g the glyph to fix.
	 * @param x the horizontal size.
	 * @param y the vertical size.
	 * @return the glyph.
	 */
	public static Glyph fixed (Glyph g, float x, float y) {
		return new Placement(g, new SuperposeLayout(new FixedLayout(FixedLayout.xAxis, x), new FixedLayout(FixedLayout.yAxis, y)));
		}

	/**
	 * Fix the size along an axis.
	 *
	 * @param g the glyph to fix.
	 * @param axis the axis that is fixed.
	 * @param size the size of the fixed axis.
	 * @return the glyph.
	 */
	public static Glyph fixedAxis (Glyph g, int axis, float size) {
		return new Placement(g, new FixedLayout(axis, size));
		}

	/**
	 * Fix the x axis.
	 *
	 * @param g the glyph to fix.
	 * @param size the size.
	 * @return the glyph.
	 */
	public static Glyph hfixed (Glyph g, float size) {
		return fixedAxis(g, FixedLayout.xAxis, size);
		}

	/**
	 * Fix the y axis.
	 *
	 * @param g the glyph to fix.
	 * @param size the size.
	 * @return the glyph.
	 */
	public static Glyph vfixed (Glyph g, float size) {
		return fixedAxis(g, FixedLayout.yAxis, size);
		}

	/**
	 * Return a glyph that allows a fixed glyph to stretch and shrink. The
	 * stretch and shrink parameters are added or subtracted from the natural
	 * size.
	 *
	 * @param g the glyph to make flexible.
	 * @param stretch the ammout of stretch.
	 * @param shrink the amount of shrink.
	 * @return the glyph.
	 */
	public static Glyph flexible (Glyph g, float stretch, float shrink) {
		return new Placement(g, new SuperposeLayout(new
		VariableLayout(VariableLayout.xAxis, stretch, shrink), new
		VariableLayout(VariableLayout.yAxis, stretch, shrink)));
		}

	/**
	 * Make a glyph stretch as much as possible.
	 *
	 * @param the glyph to stretch.
	 * @return the glyph.
	 */
	public static Glyph flexibleFil (Glyph g) {
		return flexible(g, Glue.Fil, 0F);
		}

	/**
	 * Make an axis flexible.
	 *
	 * @param g the glyph to make flexible.
	 * @param axis the axis to make flexible.
	 * @param stretch the ammout of stretch.
	 * @param shrink the amount of shrink.
	 * @return the glyph.
	 */
	public static Glyph flexibleAxis (Glyph g, int axis, float stretch, float shrink) {
		return new Placement(g, new VariableLayout(axis, stretch, shrink));
		}

	/**
	 * Make a glyph stretch horizontally.
	 *
	 * @param g the glyph to make flexible.
	 * @param stretch the ammout of stretch.
	 * @param shrink the amount of shrink.
	 * @return the glyph.
	 */
	public static Glyph hflexible (Glyph g, float stretch, float shrink) {
		return flexibleAxis(g, Glue.xAxis, stretch, shrink);
		}

	/**
	 * Make a glyph stretch vertically.
	 *
	 * @param g the glyph to make flexible.
	 * @param stretch the ammout of stretch.
	 * @param shrink the amount of shrink.
	 * @return the glyph.
	 */
	public static Glyph vflexible (Glyph g, float stretch, float shrink) {
		return flexibleAxis(g, Glue.yAxis, stretch, shrink);
		}

	/**
	 * Make a glyph that has a new natural size.
	 *
	 * @param g the glyph to change natural on.
	 * @param x the natural size in the x direction.
	 * @param y the natural size in the y direction.
	 * @return the glyph.
	 */
	public static Glyph natural (Glyph g, float x, float y) {
		return new Placement(g, new SuperposeLayout(new NaturalLayout(NaturalLayout.xAxis, x), new NaturalLayout(NaturalLayout.yAxis, y)));
		}

	/**
	 * Make a glyph that has a new natural size along an axis.
	 *
	 * @param g the glyph to change natural on.
	 * @param axis the axis to change on.
	 * @param size the natural size.
	 * @return the glyph.
	 */
	public static Glyph naturalAxis (Glyph g, int axis, float size) {
		return new Placement(g, new NaturalLayout(axis, size));
		}

	/**
	 * Make a glyph that has a new natural size along the x axis.
	 *
	 * @param g the glyph to change the natural on.
	 * @param size the natural size.
	 */
	public static Glyph hnatural (Glyph g, float size) {
		return naturalAxis(g, NaturalLayout.xAxis, size);
		}

	/**
	 * Make a glyph that has a new natural size along the y axis.
	 *
	 * @param g the glyph to change the natural on.
	 * @param size the natural size.
	 */
	public static Glyph vnatural (Glyph g, float size) {
		return naturalAxis(g, NaturalLayout.yAxis, size);
		}

	/**
	 * Add a margin to all four sides of a glyph.
	 *
	 * @param g the glyph.
	 * @param all the margin size;
	 * @return the new glyph.
	 */
	public static Glyph margin (Glyph g, float all) {
		return new Placement(g, new MarginLayout(all));
		}

	/**
	 * Add a margin to all four sides of a glyph and make it flexible.
	 *
	 * @param g the glyph.
	 * @param margin the margin size;
	 * @param stretch the stretch amount.
	 * @param shrink the shrink amount.
	 * @return the new glyph.
	 */
	public static Glyph marginFlexible (Glyph g, float margin, float stretch, float shrink) {
		return new Placement(g, new MarginLayout(margin, stretch, shrink, margin, stretch, shrink, margin, stretch, shrink, margin, stretch, shrink));
		}

	/**
	 * Add a margin to all four sides of a glyph.
	 *
	 * @param g the glyph.
	 * @param lmargin the left margin size;
	 * @param rmargin the right margin size;
	 * @param bmargin the bottom margin size;
	 * @param tmargin the top margin size;
	 * @return the new glyph.
	 */
	public static Glyph marginLrbt (Glyph g, float lmargin, float rmargin, float bmargin, float tmargin) {
		return new Placement(g, new MarginLayout(lmargin, rmargin, bmargin, tmargin));
		}

	/**
	 * Add a margin to all four sides of a glyph and make it flexible.
	 *
	 * @param g the glyph.
	 * @param lmargin the left margin size;
	 * @param rmargin the right margin size;
	 * @param bmargin the bottom margin size;
	 * @param tmargin the top margin size;
	 * @param lstretch the left stretch amount.
	 * @param rstretch the right stretch amount.
	 * @param bstretch the bottom stretch amount.
	 * @param tstretch the top stretch amount.
	 * @param lshrink the left shrink amount.
	 * @param rshrink the right shrink amount.
	 * @param bshrink the bottom shrink amount.
	 * @param tshrink the top shrink amount.
	 * @return the new glyph.
	 */
	public static Glyph marginLrbtFlexible (Glyph g,
		float lmargin, float lstretch, float lshrink,
		float rmargin, float rstretch, float rshrink,
		float bmargin, float bstretch, float bshrink,
		float tmargin, float tstretch, float tshrink ) {

		return new Placement(g, new MarginLayout(
			lmargin, lstretch, lshrink,
			rmargin, rstretch, rshrink,
			bmargin, bstretch, bshrink,
			tmargin, tstretch, tshrink));
		}

	/**
	 * Add a margin to the horizontal sides of a glyph.
	 *
	 * @param g the glyph.
	 * @param both the margin size;
	 * @return the new glyph.
	 */
	public static Glyph hmargin (Glyph g, float both) {
		return marginLrbt(g, both, both, 0F, 0F);
		}

	/**
	 * Add a margin to the horizontal sides of a glyph.
	 *
	 * @param g the glyph.
	 * @param lmargin the left margin size;
	 * @param rmargin the right margin size;
	 * @return the new glyph.
	 */
	public static Glyph hmarginLr (Glyph g, float lmargin, float rmargin) {
		return marginLrbt(g, lmargin, rmargin, 0F, 0F);
		}

	/**
	 * Add a margin to the horizontal sides of a glyph and make it flexible.
	 *
	 * @param g the glyph.
	 * @param lmargin the left margin size;
	 * @param rmargin the right margin size;
	 * @param lstretch the left stretch ammount.
	 * @param rstretch the right stretch ammount.
	 * @param lshrink the left shrink ammount.
	 * @param rshrink the right shrink ammount.
	 * @return the new glyph.
	 */
	public static Glyph hmarginLrFlexible (Glyph g, float lmargin, float lstretch, float lshrink, float rmargin, float rstretch, float rshrink) {
		return marginLrbtFlexible(g,
			lmargin, lstretch, lshrink, rmargin, rstretch, rshrink,
			0F, 0F, 0F, 0F, 0F, 0F);
		}

	/**
	 * Add a margin to the vertical sides of a glyph.
	 *
	 * @param g the glyph.
	 * @param both the margin size;
	 * @return the new glyph.
	 */
	public static Glyph vmargin (Glyph g, float both) {
		return marginLrbt(g, 0F, 0F, both, both);
		}

	/**
	 * Add a margin to the vertical sides of a glyph.
	 *
	 * @param g the glyph.
	 * @param bmargin the bottom margin size;
	 * @param tmargin the top margin size;
	 * @return the new glyph.
	 */
	public static Glyph vmarginBt (Glyph g, float bmargin, float tmargin) {
		return marginLrbt(g, 0F, 0F, bmargin, tmargin);
		}

	/**
	 * Add a margin to the vertical sides of a glyph and make it flexible.
	 *
	 * @param g the glyph.
	 * @param bmargin the bottom margin size;
	 * @param tmargin the top margin size;
	 * @param bstretch the bottom stretch ammount.
	 * @param tstretch the top stretch ammount.
	 * @param bshrink the bottom shrink ammount.
	 * @param tshrink the top shrink ammount.
	 * @return the new glyph.
	 */
	public static Glyph vmarginBtFlexible (Glyph g, float bmargin, float bstretch, float bshrink, float tmargin, float tstretch, float tshrink) {
		return marginLrbtFlexible(g,
			0F, 0F, 0F, 0F, 0F, 0F,
			bmargin, bstretch, bshrink, tmargin, tstretch, tshrink);
		}


	/**
	 * Add a margin to the left side of a glyph.
	 *
	 * @param g the glyph.
	 * @param size the margin size.
	 * @return the new glyph.
	 */
	public static Glyph lmargin (Glyph g, float size) {
		return marginLrbt(g, size, 0F, 0F, 0F);
		}

	/**
	 * Add a margin to the rignt side of a glyph.
	 *
	 * @param g the glyph.
	 * @param size the margin size.
	 * @return the new glyph.
	 */
	public static Glyph rmargin (Glyph g, float size) {
		return marginLrbt(g, 0F, size, 0F, 0F);
		}

	/**
	 * Add a margin to the bottom side of a glyph.
	 *
	 * @param g the glyph.
	 * @param size the margin size.
	 * @return the new glyph.
	 */
	public static Glyph bmargin (Glyph g, float size) {
		return marginLrbt(g, 0F, 0F, size, 0F);
		}

	/**
	 * Add a margin to the top side of a glyph.
	 *
	 * @param g the glyph.
	 * @param size the margin size.
	 * @return the new glyph.
	 */
	public static Glyph tmargin (Glyph g, float size) {
		return marginLrbt(g, 0F, 0F, 0F, size);
		}

	/**
	 * Add a margin to the left side of a glyph and make it flexible.
	 *
	 * @param g the glyph.
	 * @param natural the margin size.
	 * @param stretch the stretch amount.
	 * @param shrink the shrink amount.
	 * @return the new glyph.
	 */
	public static Glyph lmarginFlexible (Glyph g, float natural, float stretch, float shrink) {
		return marginLrbtFlexible(g,
			natural, stretch, shrink,
			0F, 0F, 0F,
			0F, 0F, 0F,
			0F, 0F, 0F);
		}

	/**
	 * Add a margin to the rignt side of a glyph and make it flexible.
	 *
	 * @param g the glyph.
	 * @param natural the margin size.
	 * @param stretch the stretch amount.
	 * @param shrink the shrink amount.
	 * @return the new glyph.
	 */
	public static Glyph rmarginFlexible (Glyph g, float natural, float stretch, float shrink) {
		return marginLrbtFlexible(g,
			0F, 0F, 0F,
			natural, stretch, shrink,
			0F, 0F, 0F,
			0F, 0F, 0F);
		}

	/**
	 * Add a margin to the bottom side of a glyph and make it flexible.
	 *
	 * @param g the glyph.
	 * @param natural the margin size.
	 * @param stretch the stretch amount.
	 * @param shrink the shrink amount.
	 * @return the new glyph.
	 */
	public static Glyph bmarginFlexible (Glyph g, float natural, float stretch, float shrink) {
		return marginLrbtFlexible(g,
			0F, 0F, 0F,
			0F, 0F, 0F,
			natural, stretch, shrink,
			0F, 0F, 0F);
		}

	/**
	 * Add a margin to the top side of a glyph and make it flexible.
	 *
	 * @param g the glyph.
	 * @param natural the margin size.
	 * @param stretch the stretch amount.
	 * @param shrink the shrink amount.
	 * @return the new glyph.
	 */
	public static Glyph tmarginFlexible (Glyph g, float natural, float stretch, float shrink) {
		return marginLrbtFlexible(g,
			0F, 0F, 0F,
			0F, 0F, 0F,
			0F, 0F, 0F,
			natural, stretch, shrink);
		}

	/**
	 * Get a box that aligns horizontally.
	 *
	 * @return the box.
	 */
	public static Glyph hbox () {
		return new Box(new SuperposeLayout(new AlignLayout(AlignLayout.yAxis), new TileLayout(TileLayout.xAxis)));
		}

	/**
	 * Get a box that aligns vertically.
	 *
	 * @return the box.
	 */
	public static Glyph vbox () {
		return new Box(new SuperposeLayout(new AlignLayout(AlignLayout.xAxis), new TileLayout(TileLayout.yAxis)));
		}

	/**
	 * Get a box that aligns horizontally. The first glyph is aligned.
	 *
	 * @return the box.
	 */
	public static Glyph hboxFirstAligned () {
		return new Box(new SuperposeLayout(new AlignLayout(AlignLayout.yAxis), new TileLayoutFirstAligned(TileLayout.xAxis)));
		}

	/**
	 * Get a box that aligns vertically. The first glyph is aligned.
	 *
	 * @return the box.
	 */
	public static Glyph vboxFirstAligned () {
		return new Box(new SuperposeLayout(new AlignLayout(AlignLayout.xAxis), new TileLayoutFirstAligned(TileLayout.yAxis)));
		}

	/**
	 * Get a box that aligns horizontally. (Untested)
	 *
	 * @param align the alignment;
	 * @return the box.
	 */
	public static Glyph hboxAlignElements (float align) {
		return new BoxAlignElements(new SuperposeLayout(new TileLayout(TileLayout.xAxis), new AlignLayout(AlignLayout.yAxis)), TileLayout.yAxis, align);
		}

	/**
	 * Get a box that aligns vertically. (Untested)
	 * @param align the alignment;
	 *
	 * @return the box.
	 */
	public static Glyph vboxAlignElements (float align) {
		return new BoxAlignElements(new SuperposeLayout(new TileLayout(TileLayout.yAxis), new AlignLayout(AlignLayout.xAxis)), TileLayout.xAxis, align);
		}

	/**
	 * Return a piece of glue.
	 *
	 * @param axis the alignment axis.
	 * @param natural the default size.
	 * @param stretch the amount the glyph can stretch.
	 * @param shrink the amount the glyph can shrink.
	 * @return the glyph.
	 */
	public static Glyph glue (int axis, float natural, float stretch, float shrink, float align) {
		return new Glue(axis, natural, stretch, shrink, align);
		}

	public static Glyph glue (Requirement r) {
		return new Glue(r);
		}

	/**
	 * Get horizontal fil. Fil can stretch infinitly, but can not shrink.
	 *
	 * @return the fil.
	 */
	public static Glyph hfil () {
		return new Glue(Glue.xAxis, 0F, Glue.Fil, 0F, 0F);
		}

	/**
	 * Get horizontal glue that can stretch infinitly.
	 *
	 * @param natural the default size.
	 * @return the glue.
	 */
	public static Glyph hglueFil (float natural) {
		return new Glue(Glue.xAxis, natural, Glue.Fil, 0F, 0F);
		}

	/**
	 * Get horizontal glue.
	 *
	 * @param natural the default size.
	 * @param stretch the amount the glue can stretch.
	 * @param shrink the amount the glue can shrink.
	 * @return the glue.
	 */
	public static Glyph hglue (float natural, float stretch, float shrink) {
		return new Glue(Glue.xAxis, natural, stretch, shrink, 0F);
		}

	/**
	 * Get horizontal glue with an alignment.
	 *
	 * @param natural the default size.
	 * @param stretch the amount the glue can stretch.
	 * @param shrink the amount the glue can shrink.
	 * @param alignment the alignement of the glue.
	 * @return the glue.
	 */
	public static Glyph hglueAligned (float natural, float stretch, float shrink, float alignment) {
		return new Glue(Glue.xAxis, natural, stretch, shrink, alignment);
		}

	/**
	 * Get a fixed horizontal space.
	 *
	 * @return the space glyph.
	 */
	public static Glyph hspace (float size) {
		return new Glue(Glue.xAxis, size, 0F, 0F, 0F);
		}

	public static Glyph vfil () {
		return new Glue(Glue.yAxis, 0F, Glue.Fil, 0F, 0F);
		}

	public static Glyph vglueFil (float natural) {
		return new Glue(Glue.yAxis, natural, Glue.Fil, 0F, 0F);
		}

	public static Glyph vglue (float natural, float stretch, float shrink) {
		return new Glue(Glue.yAxis, natural, stretch, shrink, 0F);
		}

	public static Glyph vglueAligned (float natural, float stretch, float shrink, float alignment) {
		return new Glue(Glue.yAxis, natural, stretch, shrink, alignment);
		}

	public static Glyph vspace (float size) {
		return new Glue(Glue.yAxis, size, 0F, 0F, 0F);
		}

	/**
	 * Get a glyph that places multiple glyphs on top of each other. It is
	 * helpful to use a fixed glyph to set the size after overlaying
	 * to make sure the alignment with other glyphs is correct. Overlaying
	 * glyphs will produce incorrect sizes and alignments.
	 * <p>
	 * Use append() and prepend() to add glyphs.
	 *
	 * @return the glyph.
	 */
	public static Glyph overlay () {
		return new Box(new SuperposeLayout(new AlignLayout(AlignLayout.xAxis), new AlignLayout(AlignLayout.yAxis)));
		}

	// @@ These are pretty brain dead and useless. The requirements and
	// allocation are done by the between glyph which can cause the
	// glyphs to be the same size and you will only see one glyph.

	/**
	 * Put a glyph under another glyph. (Broken)
	 *
	 * @param g the glyph.
	 * @param under the glyph underneath.
	 * @return the new glyph.
	 */
	public static Glyph back (Glyph g, Glyph under) {
		return new LayerLayout(g, under, null);
		}

	/**
	 * Put a glyph in front of another glyph. (Broken)
	 *
	 * @param g the glyph.
	 * @param over the glyph in front.
	 * @return the new glyph.
	 */
	public static Glyph front (Glyph g, Glyph over) {
		return new LayerLayout(g, null, over);
		}

	/**
	 * Put a glyph over and under another glyph. (Broken);
	 *
	 * @param g the glyph.
	 * @param under the glyph underneath.
	 * @param over the glyph in front.
	 * @return the new glyph.
	 */
	public static Glyph front (Glyph g, Glyph under, Glyph over) {
		return new LayerLayout(g, under, over);
		}

	/**
	 * A constraint that controls the the scrollable region when the view is larger than the data.
	 * @return the constraint.
	 */
    public static FullyVisibleConstraint createFullyVisibleConstraint(float usable, float align) {
	    return new FullyVisibleConstraint(usable, align);
    }

    /**
     * A viewport allows its child to be scrolled.
     *
     * @param child the child.
     * @return the viewport.
     */
    public static Viewport createViewport (Glyph child) {
        return new Viewport(child);
    }

}
