package sgraphics.layout;

// Copyright (c) 1996    Michael A. Jones

import sgraphics.*;

/**
 * A layout class that tiles along an axis.
 */
class TileLayout extends LayoutManager implements Axis {
	private int axis_;

	// These 2 variables are speed enhancements that prevent uneccessary calls to
	// requirements. It is assumed that requirements is always called before
	// allocate.
	private Requirements requirements_ = new Requirements();
	private boolean requirementsCalled_ = false;

	/**
	 * @param axis the axis to file along.
	 */
	public TileLayout (int axis) {
		axis_ = axis;
		}

    /**
     * Adds up all the mins, maxes and naturals and returns with alignemt = 0.
     */
	public void requirements (Glyph list[], Requirements requirements) {
		requirementsCalled_ = true;

		// Find out how many Glyphs there are;
		int listLength = list.length;
		int length = 0;
		while (length < listLength) {
			if (list[length] == null) {
				break;
				}
			length++;
			}


		float min = 0F;
		float max = 0F;
		float nat = 0F;
		for (int cnt = 0; cnt < length; cnt++) {
			// Get the requirement for the concerned axis.
			Requirement r = GlyphImpl.requirement(list[cnt].requirements(), axis_);
			if (r.defined) {
				max += r.maximum;
				min += r.minimum;
				nat += r.natural;
				}
			}


		// Set the requirement.
		Requirement r = GlyphImpl.requirement(requirements, axis_);
		r.maximum = max;
		r.minimum = min;
		r.natural = nat;
		// @@ Because this is zero, it is possible that the first glyph will draw
		// outside the allocation if it has an alignment along the axis.
		r.align = 0F;
		r.defined = true;

		// Copy the requirements to save. These are copied to prevent outsiders
		// from changing the requirements.
		r = GlyphImpl.requirement(requirements, xAxis);
		Requirement saved = GlyphImpl.requirement(requirements_, xAxis);
		saved.natural = r.natural;
		saved.minimum = r.minimum;
		saved.maximum = r.maximum;
		saved.align = r.align;
		r = GlyphImpl.requirement(requirements, yAxis);
		saved = GlyphImpl.requirement(requirements_, yAxis);
		saved.natural = r.natural;
		saved.minimum = r.minimum;
		saved.maximum = r.maximum;
		saved.align = r.align;
		}

	public void allocate (Glyph list[], Allocation allocation) {
		if (!requirementsCalled_)
			return;

		Allocation childAllocations[] = calculateAllocations(list, allocation, requirements_, axis_, false);
		int length = childAllocations.length;
		for (int cnt = 0; cnt < length; cnt++) {
			list[cnt].allocate(childAllocations[cnt]);
			}
		}

	public static Allocation[] calculateAllocations (Glyph list[], Allocation allocation, Requirements requirements, int axis, boolean firstAligned) {
		// Find out how many Glyphs there are;
		int listSize = list.length;
		int glyphCnt = 0;
		while (glyphCnt < listSize) {
			if (list[glyphCnt] == null)
				break;
			glyphCnt++;
			}

		Requirement r = GlyphImpl.requirement(requirements, axis);
		Area a = allocation.allocation;
		BoundingSpan givenSpan = a.span (axis);

		float length = TileLayout.computeLength(r, givenSpan);
		boolean growing = length - r.natural > 0.001;
		boolean shrinking = r.natural - length > 0.001;
		float f = GlyphImpl.computeSqueeze(r, length);
		float p = givenSpan.origin;
		Allocation childAllocations[] = new Allocation[glyphCnt];
		Sgraphics s = new Sgraphics();
		for (int cnt = 0; cnt < glyphCnt; cnt++) {
			float x1 = 0;
			float y1 = 0;
			float x2 = 0;
			float y2 = 0;
			float xalign = 0;
			float yalign = 0;
			Area area;
			r = GlyphImpl.requirement(list[cnt].requirements(), axis);
			childAllocations[cnt] = new Allocation();
			if (r.defined) {
				float cspan = r.natural;
				if (growing) {
					cspan += f * (r.maximum - r.natural);
					}
				else if (shrinking) {
					cspan -= f * (r.natural - r.minimum);
					}
				if (firstAligned && cnt == 0) {
					p -= r.align * (cspan - 1);
					}
				switch (axis) {
					case xAxis :
						x1 = p - (cspan - 1) * r.align;
						x2 = x1 + cspan - 1;
						xalign = r.align;
		        		childAllocations[cnt].allocation = new Area(x1, y1, xalign, true, x2, y2, yalign, false);
		        		childAllocations[cnt].clipping = new Area(x1, y1, xalign, true, x2, y2, yalign, false);
						break;
					case yAxis :
						y1 = p - (cspan - 1) * r.align;
						y2 = y1 + cspan - 1;
						yalign = r.align;
		        		childAllocations[cnt].allocation = new Area(x1, y1, xalign, false, x2, y2, yalign, true);
		        		childAllocations[cnt].clipping = new Area(x1, y1, xalign, false, x2, y2, yalign, true);
						break;
					}

				p += cspan;
			}
			else {
				switch (axis) {
					case xAxis :
						x1 = p;
						x2 = x1;
						xalign = 0;
	        			childAllocations[cnt].allocation = new Area(x1, y1, xalign, true, x2, y2, yalign, false);
	        			childAllocations[cnt].clipping = new Area(x1, y1, xalign, true, x2, y2, yalign, false);
						break;
					case yAxis :
						y1 = p;
						y2 = y1;
						yalign = 0;
	        			childAllocations[cnt].allocation = new Area(x1, y1, xalign, false, x2, y2, yalign, true);
	        			childAllocations[cnt].clipping = new Area(x1, y1, xalign, false, x2, y2, yalign, true);
						break;
					}
				}
			}
		return childAllocations;
		}

	private static float computeLength (Requirement r, BoundingSpan s) {
		float length = s.length;
		float s_a = s.align;
		float r_a = r.align;
		if (r_a == 0) {
			length *= (1 - s_a);
			}
		else if (r_a == 1) {
			length *= s_a;
			}
		else {
			length *= Math.min(s_a / r_a, (1 - s_a) / (1 - r_a));
			}

		return length;
		}

	public Allocation[] allocations (Glyph list[], Allocation allocation) {
		return calculateAllocations(list, allocation, requirements_, axis_, false);
	}

}
