package sgraphics.layout;

// Copyright (c) 1987-91 Stanford University
// Copyright (c) 1991-95 Silicon Graphics, Inc.
// Copyright (c) 1993-95 2001 S.A.
// Copyright (c) 1993-96 Fujitsu, Ltd.
// Copyright (c) 1994-96 Firefly Design
// Copyright (c) 1995-96 Vitria, Inc.
// Copyright (c) 1996    Michael A. Jones

import sgraphics.*;

/**
 * A layout class that tiles along an axis. The alignment of the requirements
 * will be such that it uses the alignment of the first glyph. For example:
 * if a 2 hboxesFirstAligned have several glyphs and the hboxes are put in a vbox,
 * the vbox will align the hboxes so that the first glyphs are aligned
 * by their alignment.
 */
class TileLayoutFirstAligned extends LayoutManager implements Axis {
	private int axis_;
	private Requirements requirements_;
	private boolean requirementsCalled_ = false;
	/**
	 * @param axis the axis to file along.
	 */
	public TileLayoutFirstAligned (int axis) {
		axis_ = axis;
		}

	public void requirements (Glyph list[], Requirements requirements) {
		requirementsCalled_ = true;

		// Find out how many Glyphs there are;
		int listSize = list.length;
		int length = 0;
		while (length < listSize)
			if (list[length++] == null)
				break;
		length--;

		// First get all the sizes in arrays where they can be accessed fast;
		float naturalLead = 0F;
		float minLead = 0F;
		float maxLead = 0F;
		float naturalTrail = 0F;
		float minTrail = 0F;
		float maxTrail = 0F;
		for (int cnt = 0; cnt < length; cnt++) {
			// Get the requirement for the concerned axis.
			Requirement r = GlyphImpl.requirement(list[cnt].requirements(), axis_);
			if (r.defined) {

				if (cnt == 0) {
					float a = r.align;
					float aa = 1F - a;
					naturalLead = a * r.natural;
					maxLead = a * r.maximum;
					minLead = a * r.minimum;
					naturalTrail = aa * r.natural;
					maxTrail = aa * r.maximum;
					minTrail = aa * r.minimum;
					}
				else {
					naturalTrail += r.natural;
					maxTrail += r.maximum;
					minTrail += r.minimum;
					}
				}
			}

		// Set the requirement.
		Requirement r = GlyphImpl.requirement(requirements, axis_);
		GlyphImpl.requireLeadTrail(r, naturalLead, maxLead, minLead, naturalTrail, maxTrail, minTrail);
		requirements_ = requirements;
		}

	public void allocate (Glyph list[], Allocation allocation) {
		if (!requirementsCalled_)
			return;

		Allocation childAllocations[] = TileLayout.calculateAllocations(list, allocation, requirements_, axis_, true);
		int length = childAllocations.length;
		for (int cnt = 0; cnt < length; cnt++)
			list[cnt].allocate(childAllocations[cnt]);
		}

	public Allocation[] allocations (Glyph list[], Allocation allocation) {
		return TileLayout.calculateAllocations(list, allocation, requirements_, axis_, true);
	}

}
