package sgraphics.layout;

import java.util.*;
import sgraphics.*;
import sgraphics.layout.*;
import sgraphics.util.*;

// Copyright (c) 1991-94 Silicon Graphics, Inc.
// Copyright (c) 1993-94 Fujitsu, Ltd.
// Copyright (c) 1996    Michael A. Jones

class ViewportAdjustment extends Adjustment {
    private Viewport viewport_;
    private int axis_;
    private float scrollIncrement_;
    private float pageIncrement_;
    private AdjustmentConstraint constraint_;
    private AdjustmentSettings saved_;

    public ViewportAdjustment (Viewport viewport, int axis) {
        viewport_ = viewport;
        axis_ = axis;
        scrollIncrement_ = 18F; // .25 inch
        pageIncrement_ = 0F;
    }

    public float getSmallScroll () {
        return Math.min(scrollIncrement_, pageAmount());
    }

    public void setSmallScroll (float s) {
        scrollIncrement_ = s;
    }

    public float getLargeScroll () {
        return pageAmount();
    }

    public void setLargeScroll (float p) {
        pageIncrement_ = p;
    }

    public AdjustmentSettings getSettings () {
        AdjustmentSettings s = new AdjustmentSettings();
        s.lower = viewport_.lower(axis_);
        s.length = viewport_.length(axis_);
        s.upper = s.lower + s.length;
        s.currentLower = viewport_.currentLower(axis_);
        s.currentLength = viewport_.currentLength(axis_);
        s.currentUpper = s.currentLower + s.currentLength;
        return s;
    }

    public AdjustmentConstraint getAdjustmentConstraint () {
        return constraint_;
    }

    public void setAdjustmentConstraint (AdjustmentConstraint c) {
        constraint_ = c;
    }

    public void begin () {
        saved_ = getSettings();
    }

    public void commit () {
        setChanged();
        notifyObservers();
    }

    public void cancel () {
        viewport_.scrollTo(axis_, saved_.lower);
    }

    public void scrollForward () {
        scrollTo(viewport_.currentLower(axis_) + scrollIncrement_);
    }

    public void scrollBackward () {
        scrollTo(viewport_.currentLower(axis_) - scrollIncrement_);
    }

    public void pageForward () {
        scrollTo(viewport_.currentLower(axis_) + pageAmount());
    }

    public void pageBackward () {
        scrollTo(viewport_.currentLower(axis_) - pageAmount());
    }

    public void scrollBegin () {
        float begin = viewport_.lower(axis_);
        if (constraint_ != null) {
	        constraint_.constrain(viewport_.lower(axis_), viewport_.length(axis_), viewport_.currentLength(axis_), begin);
        }
        scrollTo(begin);
    }

    public void scrollEnd () {
        float end = viewport_.lower(axis_) + viewport_.length(axis_) - viewport_.currentLength(axis_);
        if (constraint_ != null) {
	        constraint_.constrain(viewport_.lower(axis_), viewport_.length(axis_), viewport_.currentLength(axis_), end);
        }
        scrollTo(end);
    }

    public void scrollTo (float lower) {
        constrain(lower);
        if (! MathUtil.equal(lower, viewport_.currentLower(axis_), 0.001F)) {
            viewport_.scrollTo(axis_, lower);
        }
    }

    public void scrollBy (float delta) {
        scrollTo(viewport_.currentLower(axis_) + delta);
    }

    public void scaleTo(float scale) { }

    public float constrain (float c) {
        if (constraint_ != null) {
	        c = constraint_.constrain(viewport_.lower(axis_), viewport_.length(axis_), viewport_.currentLength(axis_), c);
            }
        return c;
    }

    public float pageAmount() {
        return MathUtil.equal(pageIncrement_, 0F, 0.001F) ? viewport_.currentLength(axis_) : pageIncrement_;
    }

}

/**
 * A viewport allows adjustment of the view of a glyph.
 */
public class Viewport extends MonoGlyph implements Axis {
    private ViewportAdjustment xAdjustment_ = new ViewportAdjustment(this, xAxis);
    private ViewportAdjustment yAdjustment_ = new ViewportAdjustment(this, yAxis);
    private Requirements requirements_ = new Requirements();
    private boolean allocateNatural_[] = new boolean[3];
    private float currentLower_[] = new float[3];
    private float currentLength_[] = new float[3];
    private float length_[] = new float[3];
    private float lower_[] = new float[3];
    private boolean requested_ = false;
    private Allocation allocation_ = null;

    public final static int NATURAL = 1;
    public final static int GIVEN = 2;


    public Viewport (Glyph child) {
        super(child);

        allocateNatural_[xAxis] = allocateNatural_[yAxis] = true;
        Requirement r = new Requirement();
        r.defined = true;
        r.natural = 0F;
        r.maximum = 0F;
        r.minimum = 0F;
        requirements_.x = r;
        r = new Requirement();
        r.defined = true;
        r.natural = 0F;
        r.maximum = 0F;
        r.minimum = 0F;
        requirements_.y = r;
        currentLower_[xAxis] = currentLower_[yAxis] = 0F;

        if (child != null) {
            requested_ = true;
            requirements_ = child.requirements();

	        Requirement rx = requirements_.x;
	        Requirement ry = requirements_.y;

	        currentLength_[xAxis] = length_[xAxis] = rx.defined ? rx.natural : 0F;
	        currentLength_[yAxis] = length_[yAxis] = ry.defined ? ry.natural : 0F;

	        lower_[xAxis] = rx.defined ? -(length_[xAxis] * rx.align) : 0F;
	        lower_[yAxis] = ry.defined ? -(length_[yAxis] * ry.align) : 0F;

	        if (rx.defined)
	            xAdjustment_.scrollTo(-currentLength_[xAxis] * rx.align);
	        if (ry.defined)
	            yAdjustment_.scrollTo(-currentLength_[yAxis] * ry.align);
        } else {
	        requested_ = false;
	        currentLength_[xAxis] = currentLength_[yAxis] = 0F;
	        lower_[xAxis] = lower_[yAxis] = 0F;
	        length_[xAxis] = length_[yAxis] = 0F;
        }
    }

    public Requirements requirements () {
        cacheRequirements();
        Requirements r = new Requirements();
        GlyphImpl.require(r.x, requirements_.x.natural, Glue.Fil, requirements_.x.natural, requirements_.x.align);
        GlyphImpl.require(r.y, requirements_.y.natural, Glue.Fil, requirements_.x.natural, requirements_.y.align);
        return r;
    }

    public void needResize () {
        Requirements requirements = body().requirements();
        Requirement rx = requirements.x;
        Requirement ry = requirements.y;

	    length_[xAxis] = rx.defined ? rx.natural : 0F;
	    length_[yAxis] = ry.defined ? ry.natural : 0F;

	    // A cheap trick to get the Adjustments to notify
	    // its observers.
        xAdjustment_.begin();
        xAdjustment_.commit();
        yAdjustment_.begin();
        yAdjustment_.commit();

        allocateChild();

        super.needRedraw();
    }

    /**
     * By default, a viewport allocates the natural span
     * requested by its child in bot the X and Y dimensions.
     * This method changes the mode for a given dimension.
     *
     * @param mode NATURAL or GIVEN.
     * @param axis the axis.
     */
    public void setAllocationMode (int mode, int axis) {
        allocateNatural_[axis] = (mode == NATURAL) ? true : false;
    }

    /**
     * Get the allcoation mode.
     *
     * @return NATURAL or GIVEN.
     */
    public int getAllocationMode (int axis) {
        return allocateNatural_[axis] ? NATURAL : GIVEN;
    }

    /**
     * Get the adjustment for an axis. This can be used to
     * change the scroll position and it is observable.
     *
     * @param axis the axis of the adjustment.
     * @return the adjustment.
     */
    public Adjustment getScrollAdjustment (int axis) {
        return axis == xAxis ? xAdjustment_ : yAdjustment_;
    }

    protected void scrollTo (int axis, float lower) {
        if (axis == xAxis) {
            currentLower_[xAxis] = lower;
        } else {
            currentLower_[yAxis] = lower;
        }
        needResize();
    }

    protected float lower (int axis) {
        return lower_[axis];
    }

    protected float length (int axis) {
        return length_[axis];
    }

    protected float currentLower (int axis) {
        return currentLower_[axis];
    }

    protected float currentLength (int axis) {
        return currentLength_[axis];
    }

    private void cacheRequirements () {
        if (!requested_) {
            requested_ = true;
            requirements_ = body().requirements();
        	Requirement rx = requirements_.x;
        	Requirement ry = requirements_.y;

        	lower_[xAxis] = rx.defined ? -(length_[xAxis] * rx.align) : 0F;
        	lower_[yAxis] = ry.defined ? -(length_[yAxis] * ry.align) : 0F;

        	if (rx.defined) {
        	    xAdjustment_.begin();
        	    xAdjustment_.scrollTo(-currentLength_[xAxis] * rx.align);
        	    xAdjustment_.commit();
        	}
        	if (ry.defined) {
        	    yAdjustment_.begin();
        	    yAdjustment_.scrollTo(-currentLength_[yAxis] * ry.align);
        	    yAdjustment_.commit();
        	}
        }
    }

    public void allocate (Allocation a) {
        if (a != null) {
            allocation_ = a;
            cacheRequirements();
        	BoundingSpan xspan;
        	BoundingSpan yspan;
        	float xlength;
        	float ylength;
        	Requirement rx = requirements_.x;
        	Requirement ry = requirements_.y;
        	xspan = a.allocation.span(xAxis);
        	yspan = a.allocation.span(yAxis);

        	if (allocateNatural_[xAxis]) {
        	    xlength = rx.natural;
        	} else {
        	    xlength = Math.min(  // Constrain span b/w min,max.
        	        Math.max(xspan.length, rx.minimum), rx.maximum
        	    );
        	}
        	if (allocateNatural_[yAxis]) {
        	    ylength = ry.natural;
        	} else {
        	    ylength = Math.min(
        	        Math.max(yspan.length, ry.minimum), ry.maximum
        	    );
        	}
        	if (! MathUtil.equal(xlength, length_[xAxis], 0.001F)) {
        	    float margin = xlength - length_[xAxis];
        	    lower_[xAxis] -= (margin * rx.align);
        	    length_[xAxis] = xlength;
        	}
        	if (! MathUtil.equal(ylength, length_[yAxis], 0.001F)) {
        	    float margin = ylength - length_[yAxis];
        	    lower_[yAxis] -= (margin * ry.align);
        	    length_[yAxis] = ylength;
        	}

            if (! MathUtil.equal(xspan.length, currentLength_[xAxis], 0.001F)) {
        	    float margin = xspan.length - currentLength_[xAxis];
                currentLength_[xAxis] = xspan.length;
                xAdjustment_.begin();
        	    xAdjustment_.scrollTo(currentLower_[xAxis] - (margin * rx.align));
        	    xAdjustment_.commit();
            }
            if (! MathUtil.equal(yspan.length, currentLength_[yAxis], 0.001F)) {
        	    float margin = yspan.length - currentLength_[yAxis];
                yAdjustment_.begin();
                currentLength_[yAxis] = yspan.length;
        	    yAdjustment_.scrollTo(currentLower_[yAxis] - (margin * ry.align));
        	    yAdjustment_.commit();
            }

            // This helps with resizing the screen by fixing the lower position
            // of the port when the length is too long.
            float upperError = currentLower_[xAxis] + currentLength_[xAxis] - lower_[xAxis] - length_[xAxis];
            if (upperError > 0)
                currentLower_[xAxis] -= upperError;
            if (currentLower_[xAxis] < 0)
                currentLower_[xAxis] = 0;
            upperError =  currentLower_[yAxis] + currentLength_[yAxis] - lower_[yAxis] - length_[yAxis];
            if (upperError > 0)
                currentLower_[yAxis] -= upperError;
            if (currentLower_[yAxis] < 0)
                currentLower_[yAxis] = 0;

            allocateChild();
        }
    }

    private void allocateChild () {
        if (allocation_ != null) {
            BoundingSpan xspan, yspan;
        	xspan = allocation_.allocation.span(xAxis);
        	yspan = allocation_.allocation.span(yAxis);
        	float margin;
        	margin = (length_[xAxis] - xspan.length);
        	float lowerX = xspan.begin - (margin * xspan.align) - currentLower_[xAxis];
        	float upperX = lowerX + length_[xAxis];

        	margin = (length_[yAxis] - yspan.length);
        	float lowerY = yspan.begin - (margin * yspan.align) - currentLower_[yAxis];
        	float upperY = lowerY + length_[yAxis];

            Allocation a = new Allocation();
        	a.allocation = new Area(lowerX, lowerY, xspan.align, upperX, upperY, yspan.align);
        	a.clipping = new Area(0, 0, xspan.align, currentLength_[xAxis], currentLength_[yAxis], yspan.align);
            body().allocate(a);
        }
    }
}
