package sgraphics.util;

//===================================================================
// GraphicsUtil: a class that extends the drawXXX and fillXXX
// methods of the Graphics class. In particular, it adds a Line Width
// argument to most of the drawXXX methods, a Color argument to most
// of the drawXXX and fillXXX methods, and a Font argument to
// drawString and drawChars.
//
// The latest version of the source is at
// http://www.apl.jhu.edu/~hall/java/GraphicsUtil.java, and the
// latest documentation is at
// http://www.apl.jhu.edu/~hall/java/GraphicsUtil.html.
//
// Permission to use this and/or modify this for any use is granted
// provided this notice is retained.
//
// 4/96 Marty Hall. hall@apl.jhu.edu, http://www.apl.jhu.edu/~hall
//===================================================================

import java.awt.*;
import sgraphics.Painter;

/**
 * A class that extends the draw<I>xxx</I> and fill<I>xxx</I>
 * methods of java.awt.Graphics. In particular, it adds a Line Width
 * argument to most of the draw<I>xxx</I> methods, a Color argument to most
 * of the draw<I>xxx</I> and fill<I>xxx</I> methods, and a Font argument to
 * drawString and drawChars.
 * <P>
 * Rather than including the Graphics object in a call to a constructor,
 * the methods are all static, and the Graphics is included as
 * the <I>first</I> argument to each of the methods.
 * <B>Don't forget to include it.</B>
 * <P>
 * For instance, here is how you would draw a 10-pixel wide blue
 * line from (10,10) to (200, 200) and a 5-pixel thick red
 * circle of radius 50 centered at (200, 200):
 * <PRE>
 * import GraphicsUtil
 * ...
 *    public void paint(Graphics G) {
 *      ...
 *      GraphicsUtil.drawLine(G, 10, 10, 200, 200, 10, Color.blue);
 *      GraphicsUtil.drawCircle(G, 200, 200, 50, 5, Color.red);
 *      ...
 *    }
 * </PRE>
 * The latest version of the source is at
 * <A href="http://www.apl.jhu.edu/~hall/java/GraphicsUtil.java">
 * http://www.apl.jhu.edu/~hall/java/GraphicsUtil.java</A>. The latest
 * version of the documentation is at
 * <A href="http://www.apl.jhu.edu/~hall/java/GraphicsUtil.html">
 * http://www.apl.jhu.edu/~hall/java/GraphicsUtil.html</A>.
 * <P>
 * No warranty of any kind is provided (obviously :-).
 * Permission to use and/or modify the class file or sources for any
 * purpose is granted provided this notice is retained. 4/96
 * <A href="http://www.apl.jhu.edu/~hall">Marty Hall</A>
 * <A href="mailto:hall@apl.jhu.edu">(hall@apl.jhu.edu).</A>
 *
 * @see java.awt.Graphics
 * @author Marty Hall (hall@apl.jhu.edu)
 * @version 0.9 4/3/96
 */

public class GraphicsUtil {

  //===================================================================
  /** Adds a Color argument to the drawArc method of java.awt.Graphics.
   *
   * @param G The Graphics object.
   *  Ie G.drawArc(X, Y, Width, Height, X, Y, Start_Angle, End_Angle)
   *  will get called as a result of a call to this method.
   * @param X The left side of the bounding rectangle
   * @param Y The top of the bounding rectangle
   * @param Width The width of the bounding rectangle
   * @param Height The height of the bounding rectangle
   * @param Start_Angle The beginning angle <B>in degrees.</B> 0
   *   is 3 o'clock, increasing counterclockwise.
   * @param End_Angle The ending angle in degrees.
   * @param C The color in which to draw the arc.
   */

  public static void drawArc(Painter G, float X, float Y,
			     float Width, float Height,
			     int Start_Angle, int End_Angle,
			     Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    G.drawArc(X, Y, Width, Height, Start_Angle, End_Angle);
    G.setColor(Orig);
  }

  //-------------------------------------------------------------------
  /** Adds a Color argument to the fillArc method of java.awt.Graphics.
   *
   * @param G The Graphics object.
   *  Ie G.fillArc(X, Y, Width, Height, X, Y, Start_Angle, End_Angle)
   *  will get called as a result of a call to this method.
   * @param X The left side of the bounding rectangle
   * @param Y The top of the bounding rectangle
   * @param Width The width of the bounding rectangle
   * @param Height The height of the bounding rectangle
   * @param Start_Angle The beginning angle <B>in degrees.</B>0
   *   is 3 o'clock, increasing counterclockwise.
   * @param End_Angle The ending angle in degrees.
   * @param C The color in which to draw the arc.
   */

  public static void fillArc(Painter G, float X, float Y,
			     float Width, float Height,
			     int Start_Angle, int End_Angle,
			     Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    G.fillArc(X, Y, Width, Height, Start_Angle, End_Angle);
    G.setColor(Orig);
  }

  //===================================================================
  /** Adds a Color argument to the drawChars method of java.awt.Graphics.
   *
   * @param G The Graphics object.
   * @param Chars An array of characters.
   * @param Start The index in Chars at which the string starts.
   * @param Num_Chars Number of characters to draw (starting at Start).
   * @param X The left side of the string that gets drawn
   * @param Y The <B>bottom</B> (not top) of the string.
   * @param C The color in which to draw the string.
   */

  public static void drawChars(Painter G, char[] Chars,
			       int Start, int Num_Chars,
			       float X, float Y,
			       Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    G.drawChars(Chars, Start, Num_Chars, X, Y);
    G.setColor(Orig);
  }

  /** Adds a Font argument to the drawChars method of java.awt.Graphics.
   *
   * @param G The Graphics object.
   * @param Chars An array of characters.
   * @param Start The index in Chars at which the string starts.
   * @param Num_Chars Number of characters to draw (starting at Start).
   * @param X The left side of the string that gets drawn
   * @param Y The <B>bottom</B> (not top) of the string.
   * @param F The font in which to draw the string.
   */

  public static void drawChars(Painter G, char[] Chars,
			       int Start, int Num_Chars,
			       float X, float Y,
			       Font F) {
    Font Orig = G.getFont();
    G.setFont(F);
    G.drawChars(Chars, Start, Num_Chars, X, Y);
    G.setFont(Orig);
  }

  /** Adds Font and Color arguments to the drawChars method of
   * java.awt.Graphics.
   *
   * @param G The Graphics object.
   * @param Chars An array of characters.
   * @param Start The index in Chars at which the string starts.
   * @param Num_Chars Number of characters to draw (starting at Start).
   * @param X The left side of the string that gets drawn
   * @param Y The <B>bottom</B> (not top) of the string.
   * @param F The font in which to draw the string.
   * @param C The color in which to draw the string.
   */

  public static void drawChars(Painter G, char[] Chars,
			       int Start, int Num_Chars,
			       float X, float Y,
			       Font F, Color C) {
    Font Orig_Font = G.getFont();
    G.setFont(F);
    drawChars(G, Chars, Start, Num_Chars, X, Y, C);
    G.setFont(Orig_Font);
  }

  //===================================================================
  /** Calls the drawOval method of java.awt.Graphics with a
   *  square bounding box centered at specified location with
   *  width/height of 2R.
   *
   * @param G The Graphics object.
   * @param X The x-coordinate of the center of the circle.
   * @param Y The y-coordinate of the center of the circle.
   * @param R The radius of the circle.
   */

  public static void drawCircle(Painter G, float X, float Y, float R) {
    G.drawOval(X-R, Y-R, 2*R, 2*R);
  }

  // Calling drawOval directly would save having to do X-R, 2*R calculations
  // each time, but that time is insignificant compared to the drawing time,
  // and it is easier and more extensible to use existing drawCircle method.

  // Unfortunately, drawOval calls with concentric radii do not
  // exactly fall next to each other in all cases. So occasional
  // pixels will get omitted.
  // If you knew nothing was inside your circle, you could avoid
  // this by implementing line thickness by two consecutive
  // calls to fillOval (the second using the current background color),
  // but this would require the circle drawing to be done first when
  // things are inside it, prevent overlapping circles, etc.
  // So instead 4 offset inner circles are drawn before each
  // centered inner circle.

  // Note that the radius R is to the *center* of the doughnut drawn.
  // The outside radius will be R+Line_Width/2 (rounded down).
  // Inside radius will be R-Line_Width/2 (rounded down).

  /** Draws a circle of radius R at location (X,Y) with
   *  the specified line width. Note that the radius R is to the
   *  <B>center</B> of the doughnut drawn. The outside radius will
   *  be R+Line_Width/2 (rounded down). Inside radius will be
   *  R-Line_Width/2 (rounded down).
   *
   * @param G The Graphics object.
   * @param X The x-coordinate of the center of the circle.
   * @param Y The y-coordinate of the center of the circle.
   * @param R The radius of the circle.
   * @param Line_Width Pen thickness of circle drawn.
   */

  public static void drawCircle(Painter G, float X, float Y, float R,
				float Line_Width) {
    R = R+Line_Width/2;
    for(float I=0; I<Line_Width; I+=0.5F) {
      drawCircle(G, X, Y, R);
      if ((I+0.5F)<Line_Width) {
	drawCircle(G, X+0.5F, Y, R-0.5F);
	drawCircle(G, X-0.5F, Y, R-0.5F);
	drawCircle(G, X, Y+0.5F, R-0.5F);
	drawCircle(G, X, Y-0.5F, R-0.5F);
	R = R-0.5F;
      }
    }
  }

  /** Draws a circle of radius R at location (X,Y) with
   *  the specified line width and color. Note that the radius R is to
   *  the <B>center</B> of the doughnut drawn. The outside radius will
   *  be R+Line_Width/2 (rounded down). Inside radius will be
   *  R-Line_Width/2 (rounded down).
   *
   * @param G The Graphics object.
   * @param X The x-coordinate of the center of the circle.
   * @param Y The y-coordinate of the center of the circle.
   * @param R The radius of the circle.
   * @param Line_Width Pen thickness of circle drawn.
   * @param C The color in which to draw.
   */

  public static void drawCircle(Painter G, float X, float Y, float R,
				float Line_Width, Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    drawCircle(G, X, Y, R, Line_Width);
    G.setColor(Orig);
  }

  /** Calls the drawOval method of java.awt.Graphics with a
   *  square bounding box centered at specified location with
   *  width/height of 2R. Draws in the color specified.
   *
   * @param G The Graphics object.
   * @param X The x-coordinate of the center of the circle.
   * @param Y The y-coordinate of the center of the circle.
   * @param R The radius of the circle.
   * @param C The color in which to draw.
   */

  public static void drawCircle(Painter G, float X, float Y, float R,
				Color C) {
    drawCircle(G, X, Y, R, 1F, C);
  }

  //-------------------------------------------------------------------
  /** Calls the fillOval method of java.awt.Graphics with a
   *  square bounding box centered at specified location with
   *  width/height of 2R.
   *
   * @param G The Graphics object.
   * @param X The x-coordinate of the center of the circle.
   * @param Y The y-coordinate of the center of the circle.
   * @param R The radius of the circle.
   */

  public static void fillCircle(Painter G, float X, float Y, float R) {
   G.fillOval(X-R, Y-R, 2*R, 2*R);
  }

  /** Calls the fillOval method of java.awt.Graphics with a
   *  square bounding box centered at specified location with
   *  width/height of 2R. Draws in the color specified.
   *
   * @param G The Graphics object.
   * @param X The x-coordinate of the center of the circle.
   * @param Y The y-coordinate of the center of the circle.
   * @param R The radius of the circle.
   * @param C The color in which to draw.
   */

  public static void fillCircle(Painter G, float X, float Y, float R, Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    fillCircle(G, X, Y, R);
    G.setColor(Orig);
  }

  //===================================================================
  /** Draws a line from (X1, Y1) to (X2, Y2) using the specified
   *  pen thickness.
   *
   * @param G The Graphics object.
   * @param X1 X position of start of line.
   * @param Y1 Y position of start of line.
   * @param X2 X position of end of line.
   * @param Y2 Y position of end of line.
   * @param Line_Width Thickness of line drawn.
   */

  public static void drawLine(Painter G, float X1, float Y1, float X2, float Y2,
			      float Line_Width) {
    double Angle;
    double Delta_X = (double)(X2 - X1);
    double Delta_Y = (double)(Y2 - Y1);
    double R = ((double)Line_Width)/2.0;
    double Cos_Angle, Sin_Angle, Double_I;

    if (X1 == X2)
      Angle=Math.PI;
    else
      Angle=Math.atan(Delta_Y/Delta_X)+Math.PI/2;

    Cos_Angle = Math.cos(Angle);
    Sin_Angle = Math.sin(Angle);

    X1 = X1 - (float)(R*Cos_Angle);
    X2 = X2 - (float)(R*Cos_Angle);
    Y1 = Y1 - (float)(R*Sin_Angle);
    Y2 = Y2 - (float)(R*Sin_Angle);
    for(float I=0; I<Line_Width; I+=0.5F) {
      Double_I = (double)I;
      G.drawLine(X1 + (float)(Double_I*Cos_Angle),
		 Y1 + (float)(Double_I*Sin_Angle),
		 X2 + (float)(Double_I*Cos_Angle),
		 Y2 + (float)(Double_I*Sin_Angle));

      // Drawing two parallel diagonal lines 1 pixel apart will not always
      // result in a solid diagonal line of twice the width. Instead, you often
      // get something like the following, resulting in a "gray" (1/2
      // the background color) line. So the second call fills in the
      // gap in the middle.
      //    []
      //  []  []
      //    []  []
      //      []  []
      //        []  []
      //          []  []

      if((I+1)<Line_Width)
	G.drawLine(X1 + 0.5F + (float)(Double_I*Cos_Angle),
		   Y1 +        (float)(Double_I*Sin_Angle),
		   X2 + 0.5F + (float)(Double_I*Cos_Angle),
		   Y2 +        (float)(Double_I*Sin_Angle));
    }
  }

  /** Draws a line from (X1, Y1) to (X2, Y2) using the specified
   *  pen thickness and color.
   *
   * @param G The Graphics object.
   * @param X1 X position of start of line.
   * @param Y1 Y position of start of line.
   * @param X2 X position of end of line.
   * @param Y2 Y position of end of line.
   * @param Line_Width Thickness of line drawn.
   * @param C The color in which to draw.
   */

  public static void drawLine(Painter G, float X1, float Y1, float X2, float Y2,
			      float Line_Width, Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    drawLine(G, X1, Y1, X2, Y2, Line_Width);
    G.setColor(Orig);
  }

  /** Draws a 1-pixel wide line from (X1, Y1) to (X2, Y2) using the
   * specified color.
   *
   * @param G The Graphics object.
   * @param X1 X position of start of line.
   * @param Y1 Y position of start of line.
   * @param X2 X position of end of line.
   * @param Y2 Y position of end of line.
   * @param C The color in which to draw.
   */

  public static void drawLine(Painter G, float X1, float Y1, float X2, float Y2,
			      Color C) {
    drawLine(G, X1, Y1, X2, Y2, 1, C);
  }

  //===================================================================
  /** Draws an oval in the specified bounding rectangle with
   *  the specified pen thickness. Note that the rectangle
   *  bounds the <B>center</B> (not the outside) of the oval.
   *  So the oval will really go Line_Width/2 pixels beyond bounding
   *  rectangle in all directions. Specifying a width of 1 has
   *  the identical effect to G.drawOval(Left, Top, Width, Height).
   *
   * @param G The Graphics object.
   * @param Left The left side of the bounding rectangle.
   * @param Top The y-coordinate of the top of the bounding rectangle.
   * @param Width The width of the bounding rectangle.
   * @param Height The height of the bounding rectangle.
   * @param Line_Width The pen thickness.
   */

  public static void drawOval(Painter G, float Left, float Top,
			      float Width, float Height, float Line_Width) {
    Left = Left - Line_Width/2;
    Top = Top - Line_Width/2;
    Width = Width + Line_Width;
    Height = Height + Line_Width;
    for(float I=0; I<Line_Width; I+=0.5F) {
      G.drawOval(Left, Top, Width, Height);
      if((I+0.5F)<Line_Width) {
	G.drawOval(Left,      Top,      Width-0.5F, Height-0.5F);
	G.drawOval(Left+0.5F, Top,      Width-0.5F, Height-0.5F);
	G.drawOval(Left,      Top+0.5F, Width-0.5F, Height-0.5F);
	G.drawOval(Left+0.5F, Top+0.5F, Width-0.5F, Height-0.5F);
	Left = Left + 0.5F;
	Top = Top + 0.5F;
	Width = Width - 1.0F;
	Height = Height - 1.0F;
      }
    }
  }

  /** Draws an oval in the specified bounding rectangle with
   *  the specified pen thickness and color. Note that the rectangle
   *  bounds the <B>center</B> (not the outside) of the oval.
   *  So the oval will really go Line_Width/2 pixels beyond bounding
   *  rectangle in all directions. Specifying a width of 1 has
   *  the identical effect to G.drawOval(Left, Top, Width, Height).
   *
   * @param G The Graphics object.
   * @param Left The left side of the bounding rectangle.
   * @param Top The y-coordinate of the top of the bounding rectangle.
   * @param Width The width of the bounding rectangle.
   * @param Height The height of the bounding rectangle.
   * @param Line_Width The pen thickness.
   * @param C The color in which to draw.
   */

  public static void drawOval(Painter G, float Left, float Top,
			      float Width, float Height,
			      float Line_Width, Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    drawOval(G, Left, Top, Width, Height, Line_Width);
    G.setColor(Orig);
  }

  /** Draws a 1-pixel thick oval in the specified bounding rectangle with
   *  the specified color.
   *
   * @param G The Graphics object.
   * @param Left The left side of the bounding rectangle.
   * @param Top The y-coordinate of the top of the bounding rectangle.
   * @param Width The width of the bounding rectangle.
   * @param Height The height of the bounding rectangle.
   * @param C The color in which to draw.
   */

  public static void drawOval(Painter G, float Left, float Top,
			      float Width, float Height,
			      Color C) {
    drawOval(G, Left, Top, Width, Height, 1F, C);
  }

  //-------------------------------------------------------------------
  /** Calls G.fillOval(Left, Top, Width, Height) after setting
   *  the color appropriately. Resets color after drawing.
   *
   * @param G The Graphics object.
   * @param Left The left side of the bounding rectangle.
   * @param Top The y-coordinate of the top of the bounding rectangle.
   * @param Width The width of the bounding rectangle.
   * @param Height The height of the bounding rectangle.
   * @param C The color in which to draw.
   */

  public static void fillOval(Painter G, float Left, float Top,
			      float Width, float Height,
			      Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    G.fillOval(Left, Top, Width, Height);
    G.setColor(Orig);
  }

  //===================================================================
  /** Draws a polygon in the specified color.
   * Having a drawPolygon with a line width argument would be nice,
   * but you can't just do it by drawing thick lines, since you could
   * jagged corners. Filling in those corners takes more work, so
   * is postponed. If someone wants to implement this and send
   * it to me, it would be great.
   */

  public static void drawPolygon(Painter G, float[] Xs, float[] Ys,
				 int Num_Points, Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    G.drawPolygon(Xs, Ys, Num_Points);
    G.setColor(Orig);
  }

  //-------------------------------------------------------------------
  /** Draws a solid polygon in the specified color. */

  public static void fillPolygon(Painter G, float[] Xs, float[] Ys,
				 int Num_Points, Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    G.fillPolygon(Xs, Ys, Num_Points);
    G.setColor(Orig);
  }

  //===================================================================
  /** Draws a rectangle at the specified location with the
   *  supplied pen thickness. Left/Top are the <B>center</B> of the lines
   *  drawn. Ie Width/Height are from the center of one side to the center
   *  of the other. So the inside width/heights are really
   *  Line_Width less than the values of Width and Height.
   *
   * @param G The Graphics object.
   * @param Left Center of left side edge.
   * @param Top Center of the top edge.
   * @param Width Distance from center of L side to center of R side.
   * @param Height Distance from center of top side to center of bottom side.
   * @param Line_Width Pen thickness.
   */

  public static void drawRect(Painter G, float Left, float Top,
			      float Width, float Height, float Line_Width) {
    Left = Left - Line_Width/2F;
    Top = Top - Line_Width/2F;
    Width = Width + Line_Width;
    Height = Height + Line_Width;
    for(float I=0F; I<Line_Width; I+=0.5F) {
      G.drawRect(Left, Top, Width, Height);
      Left = Left + 0.5F;
      Top = Top + 0.5F;
      Width = Width - 1F;
      Height = Height - 1F;
    }
  }

  /** Draws a rectangle at the specified location with the
   *  supplied pen thickness and color. Left/Top are the <B>center</B>
   * of the lines
   * drawn. Ie Width/Height are from the center of one side to the center
   * of the other. So the inside width/heights are really
   * Line_Width less than the values of Width and Height.
   *
   * @param G The Graphics object.
   * @param Left Center of left side edge.
   * @param Top Center of the top edge.
   * @param Width Distance from center of L side to center of R side.
   * @param Height Distance from center of top side to center of bottom side.
   * @param Line_Width Pen thickness.
   * @param C The color in which to draw.
   */

  public static void drawRect(Painter G, float Left, float Top, float Width,
			      float Height, float Line_Width, Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    drawRect(G, Left, Top, Width, Height, Line_Width);
    G.setColor(Orig);
  }

  /** Draws a 1-pixel thick rectangle at the specified location with the
   *  supplied color.
   *
   * @param G The Graphics object.
   * @param Left The x-coordinate of left side edge.
   * @param Top The y-coordinate of the top edge.
   * @param Width Width of rectangle.
   * @param Height Height of rectangle.
   * @param C The color in which to draw.
   */

  public static void drawRect(Painter G, float Left, float Top, float Width,
			      float Height, Color C) {
    drawRect(G, Left, Top, Width, Height, 1F, C);
  }

  //-------------------------------------------------------------------
  /** Calls G.fillRect(Left, Top, Width, Height) after setting
   *  the color appropriately. Resets the color when done.
   */

  public static void fillRect(Painter G, float Left, float Top,
			      float Width, float Height, Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    G.fillRect(Left, Top, Width, Height);
    G.setColor(Orig);
  }

  //===================================================================
  /** Draws a rounded rectangle at the specified location with the
   *  supplied pen thickness. Left/Top are the <B>center</B> of the lines
   *  drawn. Ie Width/Height are from the center of one side to the center
   *  of the other. So the inside width/heights are really
   *  Line_Width less than the values of Width and Height, and the
   *  outside width/heights are Line_Width more.
   *
   * @param G The Graphics object.
   * @param Left Center of left side edge.
   * @param Top Center of the top edge.
   * @param Width Distance from center of L side to center of R side.
   * @param Height Distance from center of top side to center of bottom side.
   * @param Arc_Width Horizontal diameter of arc at corners.
   * @param Arc_Height Vertical diameter of arc at corners.
   * @param Line_Width Pen thickness.
   */

   public static void drawRoundRect(Painter G, float Left, float Top,
				    float Width, float Height,
				    float Arc_Width, float Arc_Height,
				    float Line_Width) {
    Left = Left - Line_Width/2;
    Top = Top - Line_Width/2;
    Width = Width + Line_Width;
    Height = Height + Line_Width;
    for(float I=0; I<Line_Width; I+=0.5F) {
      G.drawRoundRect(Left, Top, Width, Height, Arc_Width, Arc_Height);
      if((I+0.5F)<Line_Width) {
	G.drawRoundRect(Left,      Top,      Width-0.5F, Height-0.5F,
			Arc_Width, Arc_Height);
	G.drawRoundRect(Left+0.5F, Top,      Width-0.5F, Height-0.5F,
			Arc_Width, Arc_Height);
	G.drawRoundRect(Left,      Top+0.5F, Width-0.5F, Height-0.5F,
			Arc_Width, Arc_Height);
	G.drawRoundRect(Left+0.5F, Top+0.5F, Width-0.5F, Height-0.5F,
			Arc_Width, Arc_Height);
	Left = Left + 0.5F;
	Top = Top + 0.5F;
	Width = Width - 0.5F;
	Height = Height - 0.5F;
      }
    }
  }

  /** Draws a rounded rectangle at the specified location with the
   *  supplied pen thickness and color. Left/Top are the <B>center</B>
   *  of the lines
   *  drawn. Ie Width/Height are from the center of one side to the center
   *  of the other. So the inside width/heights are really
   *  Line_Width less than the values of Width and Height, and the
   *  outside width/heights are Line_Width more.
   *
   * @param G The Graphics object.
   * @param Left Center of left side edge.
   * @param Top Center of the top edge.
   * @param Width Distance from center of L side to center of R side.
   * @param Height Distance from center of top side to center of bottom side.
   * @param Arc_Width Horizontal diameter of arc at corners.
   * @param Arc_Height Vertical diameter of arc at corners.
   * @param Line_Width Pen thickness.
   * @param C Pen color.
   */

  public static void drawRoundRect(Painter G, float Left, float Top,
				   float Width, float Height,
				   float Arc_Width, float Arc_Height,
				   float Line_Width, Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    drawRoundRect(G, Left, Top, Width, Height,
		  Arc_Width, Arc_Height, Line_Width);
    G.setColor(Orig);
  }

  /** Draws a 1-pixel wide rounded rectangle with the specified color.
   *  Same as G.drawRoundRect except for the color.
   *
   * @param G The Graphics object.
   * @param Left The x-coordinate of left edge.
   * @param Top The y-coordinate of the top edge.
   * @param Width Distance from L side to R side.
   * @param Height Distance from top side to bottom side.
   * @param Arc_Width Horizontal diameter of arc at corners.
   * @param Arc_Height Vertical diameter of arc at corners.
   * @param C Pen color.
   */

  public static void drawRoundRect(Painter G, float Left, float Top,
				   float Width, float Height,
				   float Arc_Width, float Arc_Height,
				   Color C) {
    drawRoundRect(G, Left, Top, Width, Height, Arc_Width, Arc_Height, 1F, C);
  }

  //-------------------------------------------------------------------
  /** Draws a solid rounded rectangle with the specified color.
   *  Same as G.fillRoundRect except for the color.
   *
   * @param G The Graphics object.
   * @param Left Center of left side edge.
   * @param Top Center of the top edge.
   * @param Width Distance from center of L side to center of R side.
   * @param Height Distance from center of top side to center of bottom side.
   * @param Arc_Width Horizontal diameter of arc at corners.
   * @param Arc_Height Vertical diameter of arc at corners.
   * @param C Pen color.
   */

  public static void fillRoundRect(Painter G, float Left, float Top,
				   float Width, float Height,
				   float Arc_Width, float Arc_Height,
				   Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    G.fillRoundRect(Left, Top, Width, Height, Arc_Width, Arc_Height);
    G.setColor(Orig);
  }

  //===================================================================
  /** Draws a 3D rectangle in the specified location with the given
   *  line thickness. Left/Top are the <B>center</B> of the lines
   *  drawn. Ie Width/Height are from the center of one side to the center
   *  of the other. So the inside width/heights are really
   *  Line_Width less than the values of Width and Height, and the
   *  outside width/heights are Line_Width more.
   *
   * @param G The Graphics object.
   * @param Left Center of left side edge.
   * @param Top Center of the top edge.
   * @param Width Distance from center of L side to center of R side.
   * @param Height Distance from center of top side to center of bottom side.
   * @param Is_Raised A boolean variable that determines if the right
   *        and bottom sides are shaded so as to try to make the rectangle
   *        look like it is higher than background (true) or lower (false).
   *        Works best with relatively thin lines and gray colors.
   * @param Line_Width The pen thickness.
   */

  public static void draw3DRect(Painter G, float Left, float Top,
				float Width, float Height,
				boolean Is_Raised, float Line_Width) {
    Left = Left - Line_Width/2;
    Top = Top - Line_Width/2;
    Width = Width + Line_Width;
    Height = Height + Line_Width;
    for(float I=0; I<Line_Width; I+=0.5F) {
      G.draw3DRect(Left, Top, Width, Height, Is_Raised);
      Left = Left + 0.5F;
      Top = Top + 0.5F;
      Width = Width - 1F;
      Height = Height - 1F;
    }
  }

  /** Draws a 3D rectangle in the specified location with the given
   *  line thickness and color. Left/Top are the <B>center</B> of the lines
   *  drawn. Ie Width/Height are from the center of one side to the center
   *  of the other. So the inside width/heights are really
   *  Line_Width less than the values of Width and Height, and the
   *  outside width/heights are Line_Width more.
   *
   * @param G The Graphics object.
   * @param Left Center of left side edge.
   * @param Top Center of the top edge.
   * @param Width Distance from center of L side to center of R side.
   * @param Height Distance from center of top side to center of bottom side.
   * @param Is_Raised A boolean variable that determines if the right
   *        and bottom sides are shaded so as to try to make the rectangle
   *        look like it is higher than background (true) or lower (false).
   *        Works best with relatively thin lines and gray colors.
   * @param Line_Width The pen thickness.
   * @param C The pen color.
   */
  public static void draw3DRect(Painter G, float Left, float Top,
				float Width, float Height,
				boolean Is_Raised,
				float Line_Width, Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    draw3DRect(G, Left, Top, Width, Height, Is_Raised, Line_Width);
    G.setColor(Orig);
  }

  /** Draws a 1-pixel thick 3D rectangle in the specified location with
   *  the given color.
   *
   * @param G The Graphics object.
   * @param Left The x-coordinate of left side edge.
   * @param Top The y-coordinate of the top edge.
   * @param Width Distance from L side to R side.
   * @param Height Distance from top side bottom side.
   * @param Is_Raised A boolean variable that determines if the right
   *        and bottom sides are shaded so as to try to make the rectangle
   *        look like it is higher than background (true) or lower (false).
   *        Works best with gray colors.
   * @param C The pen color.
   */

  public static void draw3DRect(Painter G, float Left, float Top,
				float Width, float Height,
				boolean Is_Raised,
				Color C) {
    draw3DRect(G, Left, Top, Width, Height, Is_Raised, 1F, C);
  }

  //-------------------------------------------------------------------

  public static void fill3DRect(Painter G, float Left, float Top,
				float Width, float Height,
				boolean Is_Raised, Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    G.fill3DRect(Left, Top, Width, Height, Is_Raised);
    G.setColor(Orig);
  }

  //===================================================================
  /** Calls G.drawString(S, X, Y) after setting the color to C.
   *  Resets the color after drawing.
   *
   * @param G The Graphics object.
   * @param S The string to be drawn.
   * @param X The left side of the string that gets drawn.
   * @param Y The <B>bottom</B> (not top) of the string.
   * @param C The color in which to draw the string.
   */

  public static void drawString(Painter G, String S, float X, float Y,
				Color C) {
    Color Orig = G.getColor();
    G.setColor(C);
    G.drawString(S, X, Y);
    G.setColor(Orig);
  }

  /** Calls G.drawString(S, X, Y) after setting the font to F.
   *  Resets the font after drawing.
   *
   * @param G The Graphics object.
   * @param S The string to be drawn.
   * @param X The left side of the string that gets drawn.
   * @param Y The <B>bottom</B> (not top) of the string.
   * @param F The font in which to draw the string.
   */

  public static void drawString(Painter G, String S, float X, float Y,
				Font F) {
    Font Orig = G.getFont();
    G.setFont(F);
    G.drawString(S, X, Y);
    G.setFont(Orig);
  }

  /** Calls G.drawString(S, X, Y) after setting the font to F
   *  and the color to C. Resets the font and color after drawing.
   *
   * @param G The Graphics object.
   * @param S The string to be drawn.
   * @param X The left side of the string that gets drawn.
   * @param Y The <B>bottom</B> (not top) of the string.
   * @param F The font in which to draw the string.
   * @param C The color in which to draw the string.
   */

  public static void drawString(Painter G, String S, float X, float Y,
				Font F, Color C) {
    Font Orig_Font = G.getFont();
    G.setFont(F);
    drawString(G, S, X, Y, C);
    G.setFont(Orig_Font);
  }

  //===================================================================
}
