package sgraphics.widget;

// Copyright (c) 1996    Michael A. Jones
// Copyright (c) 1997    Bill Petheram

import sgraphics.*;
import java.awt.*;
import java.util.*;

/**
 * A class that allows an awt scrollbar to be used as a glyph. If an Adjustment
 * is used, the user is responsible for remove the ScrollbarGlyph from the
 * adjustment with Ovservable.deleteObserver(Adjustment). Otherwise, finalize
 * will remove it.
 */
public class ScrollbarGlyph extends Scrollbar implements Viewer, Widget, Glyph, Observer {
    private WidgetImpl impl_ = new WidgetImpl(this);
    private Adjustment adjustment_ = null;
    private int orientation_;
    private Sgraphics s_ = new Sgraphics();

    /**
     * Constructs a new vertical Scrollbar.
     */
    public ScrollbarGlyph () {
	    this(VERTICAL);
	    orientation_ = VERTICAL;
    }

    /**
     * Constructs a new vertical Scrollbar.
     *
     * @param adjustment the Adjustment to observe.
     */
    public ScrollbarGlyph (Adjustment adjustment) {
	    this(VERTICAL);
	    adjustment_ = adjustment;
	    adjustment_.addObserver(this);
    	orientation_ = VERTICAL;
	    setParameters();
    }

    /**
     * Constructs a new Scrollbar with the specified orientation.
     * @param orientation either Scrollbar.HORIZONTAL or Scrollbar.VERTICAL.
     * @exception IllegalArgumentException when an illegal scrollbar orientation is given.
     */
    public ScrollbarGlyph (int orientation) {
        super(orientation);
        orientation_ = orientation;
    }

    /**
     * Constructs a new Scrollbar with the specified orientation.
     * @param orientation either Scrollbar.HORIZONTAL or Scrollbar.VERTICAL.
     * @param adjustment the Adjustment to observe.
     * @exception IllegalArgumentException when an illegal scrollbar orientation is given.
     */
    public ScrollbarGlyph (int orientation, Adjustment adjustment) {
        super(orientation);
        orientation_ = orientation;
        adjustment_ = adjustment;
	    adjustment_.addObserver(this);
	    setParameters();
    }

    /**
     * Constructs a new Scrollbar with the specified orientation,
     * value, page size,  and minumum and maximum values.
     * @param orientation either Scrollbar.HORIZONTAL or Scrollbar.VERTICAL
     * @param value the scrollbar's value
     * @param visible the size of the visible portion of the
     * scrollable area. The scrollbar will use this value when paging up
     * or down by a page.
     * @param minimum the minimum value of the scrollbar
     * @param maximum the maximum value of the scrollbar
     * @exception IllegalArgumentException when an illegal scrollbar orientation is given.
     */
    public ScrollbarGlyph (int orientation, int value, int visible, int minimum, int maximum) {
        super(orientation, value, visible, minimum, maximum);
        orientation_ = orientation;
    }

    /**
     * Constructs a new Scrollbar with the specified orientation,
     * value, page size,  and minumum and maximum values.
     * @param orientation either Scrollbar.HORIZONTAL or Scrollbar.VERTICAL
     * @param value the scrollbar's value
     * @param visible the size of the visible portion of the
     * scrollable area. The scrollbar will use this value when paging up
     * or down by a page.
     * @param minimum the minimum value of the scrollbar
     * @param maximum the maximum value of the scrollbar
     * @param adjustment the Adjustment to observe.
     * @exception IllegalArgumentException when an illegal scrollbar orientation is given.
     */
    public ScrollbarGlyph (int orientation, int value, int visible, int minimum, int maximum, Adjustment adjustment) {
        super(orientation, value, visible, minimum, maximum);
        orientation_ = orientation;
        adjustment_ = adjustment;
	    adjustment_.addObserver(this);
	    setParameters();
    }

    protected void finalize() throws Throwable {
        if (adjustment_ != null)
            adjustment_.deleteObserver(this);
    }

    private void setParameters () {
        AdjustmentSettings settings = adjustment_.getSettings();
        if (orientation_ == VERTICAL) {
            setValues(s_.convertVertical(settings.currentLower),
                s_.convertVertical(settings.currentLength),
                s_.convertVertical(settings.lower),
                s_.convertVertical(settings.upper - settings.currentLength)
                );
        }
        else {
            setValues(s_.convertHorizontal(settings.currentLower),
                s_.convertHorizontal(settings.currentLength),
                s_.convertHorizontal(settings.lower),
                s_.convertHorizontal(settings.upper - settings.currentLength)
                );
        }
        needRedraw();
    }

    public void update (Observable observable, Object arg) {
        setParameters();
    }

    public void disconnected () {
        impl_.disconnected();
    }

    public Requirements requirements () {
        return impl_.requirements();
    }

	public void allocate (Allocation allocation) {
	    impl_.allocate(allocation);
	}

    // Forward the rest of the calls.

	public void setParent (Glyph parent) {
		impl_.setParent(parent);
	}

	public Glyph getParentGlyph () {
		return impl_.getParentGlyph();
	}

    public void setCallback (ViewerCallback callback) {
        impl_.setCallback(callback);;
    }

    public void removeCallback () {
        impl_.removeCallback();
    }

	public void pick (Picker pick) {
	    impl_.pick(pick);
	}

	public boolean processPick(Picker pick, GraphicsEvent event) {
	    return impl_.processPick(pick, event);
	}

	public boolean handleEvent (GraphicsEvent event) {
        if (event.target == this && adjustment_ != null) {
            if (event.id == GraphicsEvent.SCROLL_ABSOLUTE ||
                    event.id == GraphicsEvent.SCROLL_LINE_DOWN ||
                    event.id == GraphicsEvent.SCROLL_LINE_UP ||
                    event.id == GraphicsEvent.SCROLL_PAGE_DOWN ||
                    event.id == GraphicsEvent.SCROLL_PAGE_UP) {
                adjustment_.begin();
                if (orientation_ == VERTICAL)
                    adjustment_.scrollTo(s_.convertVertical(getValue()));
                else
                    adjustment_.scrollTo(s_.convertHorizontal(getValue()));
                adjustment_.commit();
                return true;
            }
            return false;
        }
        else
            return false;
    }

	public Glyph body () {
		return impl_.body();
	}

	public void setBody (Glyph body) {
	    impl_.setBody(body);
	}

	public void append (Glyph glyph) {
	    impl_.append(glyph);
	}

	public void prepend (Glyph glyph) {
	    impl_.prepend(glyph);
	}

	public void replace (int position, Glyph glyph) {
	    impl_.replace(position, glyph);
	}

	public void swap (int position1, int position2) {
	    impl_.swap(position1, position2);
	}

	public Glyph getChild (int pos) {
	    return impl_.getChild(pos);
	}

	public void empty () {
	    impl_.empty();
	}

	public int count () {
	    return impl_.count();
	}

	public void grab (Viewer target) {
        impl_.grab(target);
	}

	public void ungrab () {
	    impl_.ungrab();
	}

	public void needRedraw () {
        impl_.needRedraw();
	}

	public void needRedraw (Area area) {
        impl_.needRedraw(area);
	}

	public void needResize () {
	    impl_.needResize();
	}

	public void draw (Painter painter) {
	    impl_.draw(painter);
	}

	public SgraphicsAdapter getSgraphicsAdapter () {
	    return impl_.getSgraphicsAdapter();
	}
}
