package sgraphics.widget;

// Copyright (c) 1996    Michael A. Jones
// Copyright (c) 1997    Bill Petheram

import sgraphics.*;
import sgraphics.layout.*;
import java.awt.*;

public class WidgetImpl extends ViewerImpl implements Widget {
    private Glyph parent_;
    private Component owner_;
    private Sgraphics s_ = new Sgraphics();
    private SgraphicsAdapter adapter_;
    private boolean added_ = false;
    private Allocation allocation_;
	private ViewerCallback callback_ = null;
	boolean grabbed_ = false;

    public WidgetImpl(Component comp) {
        super(null);
        owner_ = comp;
    }

    public void disconnected () {
        if (added_) {
            adapter_.remove(owner_);
            added_ = false;
        }
    }

    public Requirements requirements () {
	    if (!added_) {
            adapter_ = getSgraphicsAdapter();
            if (adapter_ != null) {
                owner_.hide();
                adapter_.add(owner_);
                added_ = true;
            }
        }
		Requirements r = new Requirements();
		Dimension p = owner_.preferredSize();
		Dimension m = owner_.minimumSize();

		r.x.defined = true;
		r.x.natural = s_.convertHorizontal(p.width);
		r.x.maximum = Glue.Fil;
		r.x.minimum = s_.convertHorizontal(m.width);
		r.x.align = 0;

		r.y.defined = true;
		r.y.natural = s_.convertVertical(p.height);
		r.y.maximum = Glue.Fil;
		r.y.minimum = s_.convertVertical(m.height);
		r.y.align = 0;

		return r;
    }

	public void allocate (Allocation allocation) {
	    allocation_ = allocation;
	    // Just in case we were removed and requirements was not called again.
	    if (!added_) {
            adapter_ = getSgraphicsAdapter();
            if (adapter_ != null) {
                owner_.hide();
                adapter_.add(owner_);
                added_ = true;
            }
        }

	    BoundingSpan xSpan = allocation.allocation.span(Area.xAxis);
	    BoundingSpan ySpan = allocation.allocation.span(Area.yAxis);

        owner_.reshape(
            s_.convertHorizontal(xSpan.begin),
            s_.convertHorizontal(ySpan.begin),
            s_.convertHorizontal(xSpan.length),
            s_.convertHorizontal(ySpan.length));
        owner_.show();
	}

	public void setParent (Glyph parent) {
		parent_ = parent;
	}

	public Glyph getParentGlyph () {
		return parent_;
	}

	public void pick (Picker pick) {
	    Picker p = new Picker(pick, (Viewer) owner_);

	    BoundingSpan xSpan = allocation_.allocation.span(Area.xAxis);
	    BoundingSpan ySpan = allocation_.allocation.span(Area.yAxis);

        float x1 = xSpan.begin;
        float y1 = ySpan.begin;
        float x2 = xSpan.end;
        float y2 = ySpan.end;

        // A small delta to guarentee hits when resolution independence
        // is used.
        float delta = 0.5F;

		if (p.hitType == Picker.POINT) {
			float x = p.x;
			float y = p.y;

			if (x >= x1 - delta && x <= x2 + delta && y >= y1 - delta && y <= y2 + delta) {
				p.hit((Glyph) owner_);
			}
		}
		else if (p.hitType == Picker.AREA) {
			// Check all four corners to see if any corner is in my area.
			// Top left.
			if (p.x >= x1 && p.x <= x2 && p.y >= y1 && p.y <= y2) {
				p.hit((Glyph) owner_);
				return;
			}
			// Top right.
			else if (p.x + p.width - 1 >= x1 && p.x + p.width - 1 <= x2 && p.y >= y1 && p.y <= y2) {
				p.hit((Glyph) owner_);
		        pick.attach(p);
			}
			// Bottom left.
			else if (p.x >= x1 && p.x <= x2 && p.y + p.height - 1 >= y1 && p.y + p.height - 1 <= y2) {
				p.hit((Glyph) owner_);
				return;
			}
			// Bottom right.
			else if (p.x + p.width - 1 >= x1 && p.x + p.width - 1 <= x2 && p.y + p.height - 1 >= y1 && p.y + p.height - 1 <= y2) {
				p.hit((Glyph) owner_);
				return;
			}
		}
		if (p.hit()) {
		    pick.attach(p);
		    pick.hit((Glyph) owner_);
		}
	}

    public void setCallback (ViewerCallback callback) {
        callback_ = callback;
    }

    public void removeCallback () {
        callback_ = null;
    }

    // Assume that this is a leaf viewer so don't look for
    // other viewers in the list.
	public boolean processPick (Picker pick, GraphicsEvent event) {
        return handleEvent(event);
	}

	public boolean handleEvent (GraphicsEvent evt) {
		 if (callback_ != null) {
            switch (evt.id) {

                case GraphicsEvent.MOUSE_ENTER:
                    return callback_.mouseEnter((Viewer) owner_, evt, evt.X, evt.Y);

                case GraphicsEvent.MOUSE_EXIT:
                    return callback_.mouseExit((Viewer) owner_, evt, evt.X, evt.Y);

                case GraphicsEvent.MOUSE_MOVE:
                    return callback_.mouseMove((Viewer) owner_, evt, evt.X, evt.Y);

                case GraphicsEvent.MOUSE_DOWN:
                        grab(this);
                        grabbed_ = true;
                        return callback_.mouseDown((Viewer) owner_, evt, evt.X, evt.Y);

                case GraphicsEvent.MOUSE_DRAG:
                    return callback_.mouseDrag((Viewer) owner_, evt, evt.X, evt.Y);

                case GraphicsEvent.MOUSE_UP:
                    if (grabbed_)
                        ungrab();
                    return callback_.mouseUp((Viewer) owner_, evt, evt.X, evt.Y);

                case GraphicsEvent.KEY_PRESS:
                case GraphicsEvent.KEY_ACTION:
                    return callback_.keyDown((Viewer) owner_, evt, evt.key);

                case GraphicsEvent.KEY_RELEASE:
                case GraphicsEvent.KEY_ACTION_RELEASE:
                    return callback_.keyUp((Viewer) owner_, evt, evt.key);

            }
        }
	    // This is cast to be sure the handleEvent is for the
	    // Viewer code.
		return ((Viewer) owner_).handleEvent(evt);
	}

	public void needRedraw () {
		if (parent_ != null) {
			parent_.needRedraw();
			}
		}

	public void needRedraw (Area area) {
		if (parent_ != null) {
			parent_.needRedraw();
			}
		}

	public void needResize () {
		if (parent_ != null) {
			parent_.needResize();
			}
		}

	public SgraphicsAdapter getSgraphicsAdapter () {
	    if (parent_ != null)
	        return parent_.getSgraphicsAdapter();
	    else
	        return null;
	}
}
