package sgraphics.wrapper;

// Copyright (c) 1996    Michael A. Jones

import sgraphics.*;
import sgraphics.layout.*;
import sgraphics.figure.*;
import java.awt.*;

/**
 * Border draws a three dimensional border, with a label, around a body glyph.
 */
public class LabeledOutline extends Outline {
    private Requirements rLabel_ = null;
    private Allocation allocation_;
    private Glyph body_ = null;
    private Glyph label_ = null;

    /**
     * Construct the outline.
     * @param body the body glyph.
     * @param label the label glyph. This can be any glyph.
     * @param highlight the color of the top and the left side of the body.
     * @param shadow the color of the bottom and the right side of the body.
     */
    public LabeledOutline (Glyph body, Glyph label, Color highlight, Color shadow) {
        super(highlight, shadow);

        body_ = body;
        label_ = label;

        wrapBody();
    }

	public Requirements requirements () {
		Requirements r = super.requirements();

		// Enlarge the requirements.
		Requirements newRequirements = new Requirements();
		newRequirements.x.defined = r.x.defined;
		newRequirements.x.natural = r.x.natural;
		newRequirements.x.minimum = r.x.minimum;
		newRequirements.x.maximum = r.x.maximum;
		newRequirements.x.align = r.x.align;
		newRequirements.y.defined = r.y.defined;
		newRequirements.y.natural = r.y.natural + rLabel_.y.natural * rLabel_.y.align;
		newRequirements.y.minimum = r.y.minimum + rLabel_.y.natural * rLabel_.y.align;
		newRequirements.y.maximum = r.y.maximum + rLabel_.y.natural * rLabel_.y.align;
		newRequirements.y.align = r.y.align;
		return newRequirements;
	}

    public void allocate (Allocation allocation) {
        allocation_ = allocation;
		Area area = allocation_.allocation;
		BoundingSpan xSpan = area.span(Axis.xAxis);
		BoundingSpan ySpan = area.span(Axis.yAxis);

		// Reduce the allocation.
		Allocation reducedAllocation = new Allocation();
		reducedAllocation.allocation = new Area (xSpan.begin, ySpan.begin + rLabel_.y.natural * rLabel_.y.align, xSpan.align, xSpan.end, ySpan.end, ySpan.align);
        super.allocate(reducedAllocation);
    }

    public void setBody (Glyph body) {
        body_ = body;
        wrapBody();
    }

    public Glyph body () {
        return body_;
    }

    public void setLabel (Glyph label) {
        label_ = label;
        wrapBody();
    }

    public Glyph label () {
        return label_;
    }

    // The spaceing is optimized for check boxes in a vbox.
    private void wrapBody () {
        Glyph labelBox = LayoutKit.hbox();
        labelBox.append(LayoutKit.hspace(5));

        FigureStyle style = new FigureStyle();
        if (label_ != null) {
            rLabel_ = label_.requirements();
            labelBox.append(FigureKit.rectangle(Figure.Clear, style, 0.0F, 0.0F, 1.0F, rLabel_.y.natural));
            labelBox.append(LayoutKit.valign(label_, 0.0F));
            labelBox.append(FigureKit.rectangle(Figure.Clear, style, 0.0F, 0.0F, 1.0F, rLabel_.y.natural));
        }
        Glyph bodyBox = LayoutKit.hbox();
        bodyBox.append(LayoutKit.hspace(8));
        if (body_ != null)
            bodyBox.append(body_);
        bodyBox.append(LayoutKit.hspace(8));

        Glyph vbox = LayoutKit.vbox();
        // Don't rely on the normal alignment becuase the space above and
        // below the char is affects the precise point the line will intersect.
        // Also, it may not have any alignment if it is not text.
        vbox.append(LayoutKit.valign(labelBox, 0.5F));
        // Extra space is not needed here as the 0.5 alignement will
        // create a little room.
        vbox.append(LayoutKit.valign(bodyBox, 0.0F));
        vbox.append(LayoutKit.vspace(4));
        super.setBody(vbox);
    }
}
