package sgraphics.wrapper;

// Copyright (c) 1996    Michael A. Jones

import sgraphics.*;
import sgraphics.figure.*;
import java.awt.*;


/**
 * ResizableFrame draws a black box with handles around its body when clicked by MB1.
 * Pressing MB1 on a handle will allow the user to stretch the box. Letting go
 * of MB1 will then change the size of the body to that of the box.<P>
 * Note: This viewer does not hide its children like a Viewer!<P>
 * If a child is hit the frame is not hit; therefore, the frame only toggles
 * when when clicked over an area that does not hit any children.<P>
 * After resizing (mouseUP) a FrameResizeEvent is posted to the body.
 */
public class ResizableFrame extends ViewerImpl {
    private Allocation allocation_ = null;
    private float centerX_;
    private float centerY_;
    private float oldSizeX_;
    private float oldSizeY_;
    private boolean undo_ = false;
    private boolean selected_ = false;
    private boolean showing_ = false;
    private boolean sizing_ = false;
    private Color highlight_;
    private Color background_;
    private FrameHandle sideHandles[];
    private Glyph hit_;

    private boolean forceSquare_ = false;
    private float minWidth_ = 0F;
    private float minHeight_ = 0F;
    private float maxWidth_ = 5000F;
    private float maxHeight_ = 5000F;

    private static final float handleSize = 6F;

    /**
     * @param child the child glyph.
     * @param highlight the frame color.
     * @param background the color when the frame is not showing.
     */
    public ResizableFrame (Glyph child, Color highlight, Color background) {
        super(child);
        highlight_ = highlight;
        background_ = background;
        FigureStyle style = new FigureStyle();
        style.foreground = Color.black;
        sideHandles = new FrameHandle[8];
        for (int cnt = 0; cnt < 8; cnt++) {
            sideHandles[cnt] = new FrameHandle(highlight_, background_, handleSize);
            sideHandles[cnt].setParent(this);
        }
    }

    public void allocate (Allocation a) {
        allocation_ = a;
		BoundingSpan xSpan = allocation_.allocation.span(Area.xAxis);
		BoundingSpan ySpan = allocation_.allocation.span(Area.yAxis);
		centerX_ = xSpan.begin + xSpan.length/2F;
		centerY_ = ySpan.begin + ySpan.length/2F;

        // Left
		Allocation allocation = new Allocation();
		allocation.allocation = new Area(xSpan.begin - 1F - handleSize/2, ySpan.begin + ySpan.length/2 - handleSize/2, 0F, xSpan.begin - 1F + handleSize/2, ySpan.begin + ySpan.length/2 + handleSize/2, 0F);
		sideHandles[0].allocate(allocation);

        // Top
		allocation = new Allocation();
		allocation.allocation = new Area(xSpan.begin + xSpan.length/2 - handleSize/2, ySpan.begin - 1F - handleSize/2, 0F, xSpan.begin + xSpan.length/2 + handleSize/2, ySpan.begin - 1F + handleSize/2, 0F);
		sideHandles[1].allocate(allocation);

        // Right
		allocation = new Allocation();
		allocation.allocation = new Area(xSpan.end + 1F - handleSize/2, ySpan.begin + ySpan.length/2 - handleSize/2, 0F, xSpan.end + 1F + handleSize/2, ySpan.begin + ySpan.length/2 + handleSize/2, 0F);
		sideHandles[2].allocate(allocation);

        // Bottom
		allocation = new Allocation();
		allocation.allocation = new Area(xSpan.begin + xSpan.length/2 - handleSize/2, ySpan.end + 1F - handleSize/2, 0F, xSpan.begin + xSpan.length/2 + handleSize/2, ySpan.end + 1F + handleSize/2, 0F);
		sideHandles[3].allocate(allocation);

        // Top Left
		allocation = new Allocation();
		allocation.allocation = new Area(xSpan.begin - 1F - handleSize/2, ySpan.begin - 1F - handleSize/2, 0F, xSpan.begin - 1F + handleSize/2, ySpan.begin - 1F + handleSize/2, 0F);
		sideHandles[4].allocate(allocation);

        // Top Right
		allocation = new Allocation();
		allocation.allocation = new Area(xSpan.end + 1F - handleSize/2, ySpan.begin - 1F - handleSize/2, 0F, xSpan.end + 1F + handleSize/2, ySpan.begin - 1F + handleSize/2, 0F);
		sideHandles[5].allocate(allocation);

        // Bottom Right
		allocation = new Allocation();
		allocation.allocation = new Area(xSpan.end + 1F - handleSize/2, ySpan.end + 1F - handleSize/2, 0F, xSpan.end + 1F + handleSize/2, ySpan.end + 1F + handleSize/2, 0F);
		sideHandles[6].allocate(allocation);

        // Bottom Left
		allocation = new Allocation();
		allocation.allocation = new Area(xSpan.begin - 1F - handleSize/2, ySpan.end + 1F - handleSize/2, 0F, xSpan.begin - 1F + handleSize/2, ySpan.end + 1F + handleSize/2, 0F);
		sideHandles[7].allocate(allocation);

        super.allocate(a);
    }


    public void pick (Picker pick) {
        super.pick(pick);

        // If no child hits then check our own.
        if (pick.getNext() == null) {
    		BoundingSpan xSpan = allocation_.allocation.span(Area.xAxis);
    		BoundingSpan ySpan = allocation_.allocation.span(Area.yAxis);
    		float x1 = xSpan.begin;
    		float y1 = ySpan.begin;
    		float x2 = xSpan.end;
    		float y2 = ySpan.end;

    		if (pick.hitType == Picker.POINT) {
    			float x = pick.x;
    			float y = pick.y;

                // Use a new picker because we only want one hit to show up in the list.
                Picker p = new Picker(pick, this);
                if (selected_) {
                    hit_ = null;
                    if (x >= x1 && x <= x2 && y >= y1 && y <= y2)
        				hit_ = this;;
                    for (int cnt = 0; cnt < 8; cnt++) {
                        sideHandles[cnt].pick(p);
                        if (p.getNext() != null) {
                            hit_ = sideHandles[cnt];
                            break;
                        }
                    }

            	    // If there is a hit, get the last hit.
            	    if (hit_ != null) {
            	        Picker chainedPicker = new Picker(pick, this);
            	        chainedPicker.hit(this);
            	        pick.attach(chainedPicker);
            	    }
        	    }
                else {
                    if (x >= x1 && x <= x2 && y >= y1 && y <= y2) {
                        super.pick(pick);
                        if (pick.getNext() == null) {
                            hit_ = this;
                	        Picker chainedPicker = new Picker(pick, this);
                	        chainedPicker.hit(this);
                	        pick.attach(chainedPicker);
            	        }
        			}
        	    }
    		}
        }
    }

    public void draw (Painter painter) {
		if (allocation_ != null) {
			BoundingSpan xSpan = allocation_.allocation.span(Area.xAxis);
			BoundingSpan ySpan = allocation_.allocation.span(Area.yAxis);
			// @@ For some reason clipping screws up when drawing a circle.
			//painter.clipRect(new Area(xSpan.begin - (handleSize/2F + 1F), ySpan.begin - (handleSize/2F + 1F), 0F, xSpan.end + (handleSize/2F + 1F), ySpan.end + (handleSize/2F + 1F), 0F));
            if (selected_ && showing_) {
                super.draw(painter);
                painter.setColor(highlight_);
			    painter.drawRect(xSpan.begin - 1F, ySpan.begin - 1F, xSpan.length + 1F, ySpan.length + 1F);
                for (int cnt = 0; cnt < 8; cnt++) {
                    sideHandles[cnt].activate();
                    sideHandles[cnt].draw(painter);
                }
            }
            else {
                super.draw(painter);
            }
			// Draw the box around the body even though it is outside the allocation.
		}
    }

    // @@ Enhance so that both a down and up are required. Down wont work
    // because then you can drag a handle.
	public boolean mouseUp (GraphicsEvent evt, float x, float y) {
	    if (hit_ instanceof FrameHandle) {
	        if (sizing_ && !showing_) {
	            SgraphicsAdapter adapter = getSgraphicsAdapter();
                Painter painter = adapter.getPainter();
                painter.setXORMode(Color.white);
                painter.drawRect(Math.round(centerX_ - oldSizeX_), Math.round(centerY_ - oldSizeY_), Math.round(2F * oldSizeX_), Math.round(2F * oldSizeY_));
                adapter.refresh();

	            showing_ = true;
	            sizing_ = false;

	            // Post to the child.
        		BoundingSpan xSpan = allocation_.allocation.span(Area.xAxis);
        		BoundingSpan ySpan = allocation_.allocation.span(Area.yAxis);
        		// A place that will generate a hit.
        		float xCenter = xSpan.begin + (xSpan.end - xSpan.begin) / 2;
        		float yCenter = ySpan.begin + (ySpan.end - ySpan.begin) / 2;

	            adapter.postEvent(new FrameResizeEvent(body(), oldSizeX_ * 2F - 1, oldSizeY_ * 2F - 1, xCenter, yCenter));
	            return true;
	        }
	    }
	    if (evt.modifiers == GraphicsEvent.SHIFT_MASK) {
	        if (selected_) {
	            disableHandles();
	            return true;
	        }
	    }
	    else {
	        if (!selected_) {
	            enableHandles();
	            return true;
	        }
	    }
		return false;
	}

    public boolean mouseDrag (GraphicsEvent evt, float x, float y) {
        if (!selected_ || !(hit_ instanceof FrameHandle))
            return false;
        int handle = 0;
        if (hit_ instanceof FrameHandle)
            for (handle = 0; handle < 8; handle++)
                if (sideHandles[handle] == (FrameHandle) hit_ && grabbing())
                    break;

        if (hit_ instanceof FrameHandle && handle < 8) {
            if (!sizing_) {
                sizing_ = true;
                showing_ = false;
                undo_ = false;
                needRedraw();
            }
            else {
                SgraphicsAdapter adapter = getSgraphicsAdapter();
                Painter painter = adapter.getPainter();
                // Figure out how to draw the box.

                // How far is the cursor from the center?
                // @@ This should be restored when everybodys XOR works!
                // painter.setXORMode(highlight_);
                painter.setXORMode(Color.white);
                float xdis = 0;
                float ydis = 0;
                // If handling a corner
                if (handle > 3) {
                    xdis = Math.abs(centerX_ - (float) x);
                    ydis = xdis;
                } else if (handle == 0 || handle == 2) {
                    xdis = Math.abs(centerX_ - (float) x);
			        BoundingSpan ySpan = allocation_.allocation.span(Area.yAxis);
                    ydis = Math.abs(ySpan.length/2F);
                    if (forceSquare_)
                        ydis = xdis;
                } else if (handle == 1 || handle == 3) {
			        BoundingSpan xSpan = allocation_.allocation.span(Area.xAxis);
                    xdis = Math.abs(xSpan.length/2F);
                    ydis = Math.abs(centerY_ - (float) y);
                    if (forceSquare_)
                        xdis = ydis;
                }
                if (xdis * 2 - 1 < minWidth_)
                    xdis = (minWidth_ + 1) / 2;
                if (ydis * 2 - 1 < minHeight_)
                    ydis = (minHeight_ + 1) / 2;
                if (xdis * 2 - 1 > maxWidth_)
                    xdis = (maxWidth_ + 1) / 2;
                if (ydis * 2 - 1 > maxHeight_)
                    ydis = (maxHeight_ + 1) / 2;
                if (undo_)
                    painter.drawRect(Math.round(centerX_ - oldSizeX_), Math.round(centerY_ - oldSizeY_), Math.round(2F * oldSizeX_), Math.round(2F * oldSizeY_));
                painter.drawRect(Math.round(centerX_ - xdis), Math.round(centerY_ - ydis), Math.round(2F * xdis), Math.round(2F * ydis));
                adapter.refresh();
                oldSizeX_ = xdis;
                oldSizeY_ = ydis;
                painter.setPaintMode();
                undo_ = true;
            }
            return true;
        }
        return false;
    }

    /**
     * Enable the handles.
     */
	public void enableHandles () {
	    if (!selected_) {
	        selected_ = true;
	        showing_ = true;
	        needRedraw();
		}
	}

    /**
     * Disable the handles.
     */
	public void disableHandles () {
	    if (selected_) {
	        selected_ = false;
	        showing_ = false;
	        needRedraw();
		}
	}

	/**
	 * Constrain the ammount the outline can grow or shrink.
	 *
	 * @param minWidth the minimum width.
	 * @param maxWidth the maximum width.
	 * @param minHeight the maximum height.
	 * @param maxHeight the maximum height.
	 */
	public void setConstraints (float minWidth, float maxWidth, float minHeight, float maxHeight) {
	    minWidth_ = minWidth;
	    minHeight_ = minHeight;
	    maxWidth_ = maxWidth;
	    maxHeight_ = maxHeight;
	}

	/**
	 * Make the outline always square no matter where the mouse goes.
	 */
	public void lockSides () {
	    forceSquare_ = true;
	}

	/**
	 * Allow the outline to be rectangle.
	 */
	public void unlockSides () {
	    forceSquare_ = false;
	}

}